@@ -1,21 +1,17 @@
 
 import os
+import re
 import py
 from .urlutil import DistURL
 from .vendor._description_utils import processDescription
-import hashlib
-from .urlutil import sorted_by_version
+from .urlutil import sorted_by_version, get_latest_version
+from .auth import crypt_password, verify_password
+from .validation import validate_metadata, normalize_name
 
 import logging
 
 log = logging.getLogger(__name__)
 
-def getpwhash(password, salt):
-    hash = hashlib.sha256()
-    hash.update(salt)
-    hash.update(password)
-    return hash.hexdigest()
-
 def run_passwd(db, user):
     if not db.user_exists(user):
         log.error("user %r not found" % user)
@@ -46,6 +42,14 @@ class DB:
         self.xom = xom
         self.keyfs = xom.keyfs
 
+    def is_empty(self):
+        userlist = self.user_list()
+        if len(userlist) != 1 or "root" not in userlist:
+            return False
+        userconfig = self.user_get("root")
+        rootindexes = list(userconfig.get("indexes", []))
+        return rootindexes == ["pypi"]
+
     def user_create(self, user, password, email=None):
         with self.keyfs.USER(user=user).update() as userconfig:
             self._setpassword(userconfig, user, password)
@@ -53,6 +57,14 @@ class DB:
                 userconfig["email"] = email
             log.info("created user %r with email %r" %(user, email))
 
+    def _user_set(self, user, newuserconfig):
+        with self.keyfs.USER(user=user).update() as userconfig:
+            if "indexes" not in newuserconfig:
+                newuserconfig["indexes"] = userconfig.get("indexes", {})
+            userconfig.clear()
+            userconfig.update(newuserconfig)
+            log.info("internal: set user information %r", user)
+
     def user_modify(self, user, password=None, email=None):
         with self.keyfs.USER(user=user).update() as userconfig:
             modified = []
@@ -65,8 +77,9 @@ class DB:
             log.info("modified user %r: %s" %(user, ", ".join(modified)))
 
     def _setpassword(self, userconfig, user, password):
-        userconfig["pwsalt"] = salt = os.urandom(16).encode("base_64")
-        userconfig["pwhash"] = hash = getpwhash(password, salt)
+        salt, hash = crypt_password(password)
+        userconfig["pwsalt"] = salt
+        userconfig["pwhash"] = hash
         log.info("setting password for user %r", user)
 
     def user_delete(self, user):
@@ -84,16 +97,18 @@ class DB:
             return False
         salt = userconfig["pwsalt"]
         pwhash = userconfig["pwhash"]
-        if getpwhash(password, salt) == pwhash:
+        if verify_password(password, pwhash, salt):
             return pwhash
         return None
 
-    def user_get(self, user):
+    def user_get(self, user, credentials=False):
         d = self.keyfs.USER(user=user).get()
-        if d:
+        if not d:
+            return d
+        if not credentials:
             del d["pwsalt"]
             del d["pwhash"]
-            d["username"] = user
+        d["username"] = user
         return d
 
     def _get_user_and_index(self, user, index=None):
@@ -216,6 +231,11 @@ class DB:
         self.index_create(user, index, **kw)
         return self.getstage(user, index)
 
+class ProjectInfo:
+    def __init__(self, stage, name):
+        self.name = name
+        self.stage = stage
+
 
 class PrivateStage:
     metadata_keys = """
@@ -258,13 +278,50 @@ class PrivateStage:
             results.append((stage, stage_result))
         return results
 
+    def log_info(self, *args):
+        log.info("%s: %s" % (self.name, args[0]), *args[1:])
     #
     # registering project and version metadata
     #
     #class MetadataExists(Exception):
     #    """ metadata exists on a given non-volatile index. """
 
+    class RegisterNameConflict(Exception):
+        """ a conflict while trying to register metadata. """
+
+    def get_project_info(self, name):
+        """ return first matching project info object for the given name
+        or None if no project exists. """
+        for stage, res in self.op_with_bases("get_project_info", name=name):
+            if res is not None:
+                return res
+
+    def get_project_info_perstage(self, name):
+        """ return normalized name for the given name or None
+        if no project exists. """
+        names = self.getprojectnames_perstage()
+        norm2name = dict([(normalize_name(x), x) for x in names])
+        realname = norm2name.get(normalize_name(name), None)
+        if realname:
+            return ProjectInfo(self, realname)
+
     def register_metadata(self, metadata):
+        """ register metadata.  Raises ValueError in case of metadata
+        errors. """
+        validate_metadata(metadata)
+        name = metadata["name"]
+        version = metadata["version"]
+        # check if the project exists already under its normalized
+        info = self.get_project_info(name)
+        if info:
+            log.info("got project info with name %r" % info.name)
+        else:
+            log.debug("project %r does not exist, good", name)
+        if info is not None and info.name != name:
+            raise self.RegisterNameConflict(info)
+        self._register_metadata(metadata)
+
+    def _register_metadata(self, metadata):
         name = metadata["name"]
         version = metadata["version"]
         key = self.keyfs.PROJCONFIG(user=self.user, index=self.index, name=name)
@@ -275,6 +332,7 @@ class PrivateStage:
             #        name, version, self.name))
             versionconfig = projectconfig.setdefault(version, {})
             versionconfig.update(metadata)
+            self.log_info("store_metadata %s-%s", name, version)
         desc = metadata.get("description")
         if desc:
             html = processDescription(desc)
@@ -298,11 +356,11 @@ class PrivateStage:
         with key.locked_update() as projectconfig:
             if version not in projectconfig:
                 return False
-            log.info("deleting version %r of project %r", version, name)
+            self.log_info("deleting version %r of project %r", version, name)
             del projectconfig[version]
         # XXX race condition if concurrent addition happens
         if not projectconfig:
-            log.info("no version left, deleting project %r", name)
+            self.log_info("no version left, deleting project %r", name)
             key.delete()
         return True
 
@@ -320,8 +378,18 @@ class PrivateStage:
             user=self.user, index=self.index, name=name)
 
     def get_metadata(self, name, version):
-        projectconfig = self.get_projectconfig(name)
-        return projectconfig.get(version)
+        # on win32 we need to make sure that we only return
+        # something if we know about the exact name, not a
+        # case-different one
+        info = self.get_project_info(name)
+        if info and info.name == name:
+            projectconfig = self.get_projectconfig(name)
+            return projectconfig.get(version)
+
+    def get_metadata_latest(self, name):
+        versions = self.get_projectconfig(name)
+        maxver = get_latest_version(versions)
+        return self.get_metadata(name, maxver.string)
 
     def get_projectconfig_perstage(self, name):
         key = self.keyfs.PROJCONFIG(user=self.user, index=self.index, name=name)
@@ -374,7 +442,7 @@ class PrivateStage:
         files = []
         for verdata in projectconfig.values():
             files.extend(
-                map(self.xom.releasefilestore.getentry,
+                map(self.xom.filestore.getentry,
                     verdata.get("+files", {}).values()))
         return files
 
@@ -386,45 +454,51 @@ class PrivateStage:
             all_names.update(names)
         return sorted(all_names)
 
-    def getprojectnames_perstage(self):
-        return self.keyfs.PROJCONFIG.listnames("name",
-                    user=self.user, index=self.index)
 
-    def store_releasefile(self, filename, content):
+    def getprojectnames_perstage(self):
+        names = self.keyfs.PROJCONFIG.listnames("name",
+                        user=self.user, index=self.index)
+        # on case insensitive filesystems we can't be sure
+        # we have case-sensitive names so we do a slow
+        # iteration over all projectconfig files
+        realnames = set()
+        for name in names:
+            projectconfig = self.get_projectconfig_perstage(name)
+            for metadata in projectconfig.values():
+                realnames.add(metadata.get("name", name))
+        return list(realnames)
+
+
+    class MissesRegistration(Exception):
+        """ store_releasefile requires pre-existing release metadata. """
+
+    def store_releasefile(self, filename, content, last_modified=None):
         name, version = DistURL(filename).pkgname_and_version
+        info = self.get_project_info(name)
+        name = getattr(info, "name", name)
+        if not self.get_metadata(name, version):
+            raise self.MissesRegistration(name, version)
+        log.debug("project name of %r is %r", filename, name)
         key = self.keyfs.PROJCONFIG(user=self.user, index=self.index, name=name)
         with key.locked_update() as projectconfig:
             verdata = projectconfig.setdefault(version, {})
             files = verdata.setdefault("+files", {})
             if not self.ixconfig.get("volatile") and filename in files:
                 return 409
-            entry = self.xom.releasefilestore.store(self.user, self.index,
-                                                    filename, content)
+            entry = self.xom.filestore.store(self.user, self.index,
+                                filename, content, last_modified=last_modified)
             files[filename] = entry.relpath
-            log.info("%s: stored releasefile %s", self.name, entry.relpath)
+            self.log_info("store_releasefile %s", entry.relpath)
             return entry
 
     def store_doczip(self, name, content):
+        """ unzip doc content for the specified "name" project. """
         assert content
-        key = self.keyfs.STAGEDOCS(user=self.user, index=self.index, name=name)
+        key = self._doc_key(name)
 
-        # XXX collission checking
-        #
-        unzipfile = py.std.zipfile.ZipFile(py.io.BytesIO(content))
-
-        # XXX locking?
+        # XXX locking (unzipping could happen concurrently in theory)
         tempdir = self.keyfs.mkdtemp(name)
-        members = unzipfile.namelist()
-        for name in members:
-            fpath = tempdir.join(name, abs=True)
-            if not fpath.relto(tempdir):
-                raise ValueError("invalid path name:" + name)
-            if name.endswith(os.sep):
-                fpath.ensure(dir=1)
-            else:
-                fpath.dirpath().ensure(dir=1)
-                with fpath.open("wb") as f:
-                    f.write(unzipfile.read(name))
+        unzip_to_dir(content, tempdir)
         keypath = key.filepath
         if keypath.check():
             old = keypath.new(basename="old-" + keypath.basename)
@@ -435,3 +509,50 @@ class PrivateStage:
             keypath.dirpath().ensure(dir=1)
             tempdir.move(keypath)
         return keypath
+
+    def get_doczip(self, name):
+        """ get zip file content of the current docs or None. """
+        key = self._doc_key(name)
+        basedir = key.filepath
+        if not basedir.check():
+            return
+        content = create_zipfile(basedir)
+        return content
+
+    def _doc_key(self, name):
+        return self.keyfs.STAGEDOCS(user=self.user, index=self.index,
+                                    name=name)
+
+
+def create_zipfile(basedir):
+    f = py.io.BytesIO()
+    zip = py.std.zipfile.ZipFile(f, "w")
+    _writezip(zip, basedir)
+    zip.close()
+    return f.getvalue()
+
+def _writezip(zip, basedir):
+    for p in basedir.visit():
+        if p.check(dir=1):
+            if not p.listdir():
+                path = p.relto(basedir) + "/"
+                zipinfo = py.std.zipfile.ZipInfo(path)
+                zip.writestr(zipinfo, "")
+        else:
+            path = p.relto(basedir)
+            zip.writestr(path, p.read("rb"))
+
+def unzip_to_dir(content, basedir):
+    unzipfile = py.std.zipfile.ZipFile(py.io.BytesIO(content))
+    members = unzipfile.namelist()
+    for name in members:
+        fpath = basedir.join(name, abs=True)
+        if not fpath.relto(basedir):
+            raise ValueError("invalid path name:" + name)
+        if name.endswith(os.sep) or name[-1] == "/":
+            fpath.ensure(dir=1)
+        else:
+            fpath.dirpath().ensure(dir=1)
+            with fpath.open("wb") as f:
+                f.write(unzipfile.read(name))
+
