@@ -17,20 +17,20 @@ from .types import propmapping
 from .urlutil import DistURL, joinpath
 from pkg_resources import Requirement
 
+from .validation import normalize_name, safe_name
+from .db import ProjectInfo
+
 from logging import getLogger
 assert __name__ == "devpi_server.extpypi"
 log = getLogger(__name__)
 
 CONCUCCRENT_CRAWL = False
-ALLOWED_ARCHIVE_EXTS = ".egg .whl .tar.gz .tar.bz2 .tar .tgz .zip".split()
-ARCHIVE_SCHEMES = ("http", "https")
 
 
 class IndexParser:
 
     def __init__(self, projectname):
-        self.projectname_raw = projectname
-        self.projectname = Requirement.parse(projectname).project_name.lower()
+        self.projectname = normalize_name(projectname)
         self.basename2link = {}
         self.crawllinks = set()
         self.egglinks = []
@@ -55,10 +55,11 @@ class IndexParser:
         seen = set()
         for link in p.links:
             newurl = DistURL(link.url)
+            if not newurl.is_valid_http_url():
+                continue
             eggfragment = newurl.eggfragment
             if scrape and eggfragment:
-                filename = eggfragment.replace("_", "-")
-                if filename.lower().startswith(self.projectname + "-"):
+                if normalize_name(eggfragment).startswith(self.projectname):
                     # XXX seems we have to maintain a particular
                     # order to keep pip/easy_install happy with some
                     # packages (e.g. nose)
@@ -68,34 +69,17 @@ class IndexParser:
                     log.debug("skip egg link %s (projectname: %s)",
                               newurl, self.projectname)
                 continue
-            if is_archive_of_project(newurl, self.projectname):
-                self._mergelink_ifbetter(newurl)
+            if newurl.is_archive_of_project(self.projectname):
                 seen.add(newurl.url)
+                self._mergelink_ifbetter(newurl)
                 continue
         if scrape:
             for link in p.rel_links():
                 if link.url not in seen:
                     disturl = DistURL(link.url)
-                    if disturl._parsed.scheme in ARCHIVE_SCHEMES:
+                    if disturl.is_valid_http_url():
                         self.crawllinks.add(disturl)
 
-def is_archive_of_project(newurl, targetname):
-    if newurl._parsed.scheme not in ARCHIVE_SCHEMES:
-        log.warn("url has unsupported scheme %r", newurl)
-        return False
-    nameversion, ext = newurl.splitext_archive()
-    parts = re.split(r'-\d+', nameversion)
-    projectname = parts[0]
-    if not projectname:
-        return False
-    try:
-        pname = Requirement.parse(projectname).project_name.lower()
-    except ValueError:
-        return False
-    #log.debug("checking %s, projectname %r, nameversion %s", newurl, self.projectname, nameversion)
-    return (len(parts) > 1 and ext.lower() in ALLOWED_ARCHIVE_EXTS and
-            pname == targetname)
-
 def parse_index(disturl, html, scrape=True):
     if not isinstance(disturl, DistURL):
         disturl = DistURL(disturl)
@@ -160,40 +144,51 @@ def perform_crawling(extdb, result, numthreads=10):
         for t in threads:
             t.join()
 
-class ExtDB:
-    name = "root/pypi"
-    ixconfig = dict(bases=(), volatile=False)
 
-    def __init__(self, xom):
-        self.keyfs = xom.keyfs
-        self.httpget = xom.httpget
-        self.releasefilestore = xom.releasefilestore
-        self.xom = xom
+def invalidate_on_version_change(basedir):
+    verfile = basedir.join(".mirrorversion")
+    if not verfile.check():
+        ver = "0"
+    else:
+        ver = verfile.read()
+    if ver != ExtDB.VERSION:
+        if basedir.check():
+            log.info("version format change: removing root/pypi state")
+            basedir.remove()
+    verfile.dirpath().ensure(dir=1)
+    verfile.write(ExtDB.VERSION)
+
+class ExtDB:
+    VERSION = "1"
+    name = "root/pypi"
+    ixconfig = dict(bases=(), volatile=False, type="mirror")
+
+    def __init__(self, keyfs, httpget, filestore, proxy):
+        self.keyfs = keyfs
+        self.httpget = httpget
+        self.filestore = filestore
+        invalidate_on_version_change(keyfs.basedir.join("root", "pypi"))
+        self.init_pypi_mirror(proxy)
 
     def getcontained(self):
         return self.keyfs.PYPILINKS.listnames("name")
 
     def getprojectnames(self):
         """ return list of all projects which have been served. """
-        log.info("slow access (big reply) to full extpypi simple list")
-        if hasattr(self, "name2serials"):
-            return sorted(x for x in self.name2serials if x[0] != ".")
-        log.warn("pypi.python.org's simple page not cached yet")
-        return sorted(self.keyfs.PYPILINKS.listnames("name"))
+        return sorted(self.name2serials)
 
     getprojectnames_perstage = getprojectnames
 
-    def _dump_projectlinks(self, projectname, dumplist, serial):
-        newlist = [serial] + dumplist
-        self.keyfs.PYPILINKS(name=projectname).set(newlist)
+    def _dump_project_cache(self, projectname, dumplist, serial):
+        normname = normalize_name(projectname)
+        data = {"serial": serial,
+                "entrylist": dumplist,
+                "projectname": projectname}
+        self.keyfs.PYPILINKS(name=normname).set(data)
 
-    def _load_projectlinks(self, projectname):
-        res = self.keyfs.PYPILINKS(name=projectname).get(None)
-        if res:
-            serial = res.pop(0)
-            assert isinstance(serial, int)
-            return serial, res
-        return None, None
+    def _load_project_cache(self, projectname):
+        normname = normalize_name(projectname)
+        return self.keyfs.PYPILINKS(name=normname).get(None)
 
     def getreleaselinks(self, projectname, refresh=0):
         """ return all releaselinks from the index and referenced scrape
@@ -207,37 +202,49 @@ class ExtDB:
 
         """
         assert not isinstance(refresh, bool), repr(refresh)
-        serial, res = self._load_projectlinks(projectname)
-        if res is not None and serial >= refresh:
-            return [self.releasefilestore.getentry(relpath) for relpath in res]
+        cache = self._load_project_cache(projectname)
+        if cache is not None and cache["serial"] >= refresh:
+            return [self.filestore.getentry(relpath)
+                        for relpath in cache["entrylist"]]
 
-        # XXX short cut 404 if self.name2serials is not empty and
-        # does not contain the projectname (watch out case-sensitivity)
-
-        url = PYPIURL_SIMPLE + projectname + "/"
+        info = self.get_project_info(projectname)
+        if not info:
+            return 404
+        url = PYPIURL_SIMPLE + info.name + "/"
         log.debug("visiting index %s", url)
         response = self.httpget(url, allow_redirects=True)
         if response.status_code != 200:
             return response.status_code
+        real_projectname = response.url.strip("/").split("/")[-1]
+        assert real_projectname == info.name
         serial = int(response.headers["X-PYPI-LAST-SERIAL"])
         if not isinstance(refresh, bool) and isinstance(refresh, int):
             if serial < refresh:
                 log.warn("%s: pypi returned serial %s, expected %s",
-                         projectname, serial, refresh)
+                         real_projectname, serial, refresh)
                 return -2  # the page we got is not fresh enough
-        log.debug("%s: got response with serial %s" % (projectname, serial))
+        log.debug("%s: got response with serial %s" %
+                  (real_projectname, serial))
         assert response.text is not None, response.text
         result = parse_index(response.url, response.text)
         perform_crawling(self, result)
         releaselinks = list(result.releaselinks)
-        entries = [self.releasefilestore.maplink(link, refresh=refresh)
+        entries = [self.filestore.maplink(link, refresh=refresh)
                         for link in releaselinks]
         dumplist = [entry.relpath for entry in entries]
-        self._dump_projectlinks(projectname, dumplist, serial)
+        self._dump_project_cache(real_projectname, dumplist, serial)
         return entries
 
     getreleaselinks_perstage = getreleaselinks
 
+    def get_project_info(self, name):
+        norm_name = normalize_name(name)
+        name = self.normname2name.get(norm_name, norm_name)
+        if name in self.name2serials:
+            return ProjectInfo(self, name)
+
+    get_project_info_perstage = get_project_info
+
     def op_with_bases(self, opname, **kw):
         return [(self, getattr(self, opname)(**kw))]
 
@@ -264,56 +271,75 @@ class ExtDB:
         return html.div("please refer to description on remote server ",
             html.a(link, href=link)).unicode(indent=2)
 
-    MIRRORVERSION = 1
+    def init_pypi_mirror(self, proxy):
+        """ initialize pypi mirror if no mirror state exists. """
+        self.proxy = proxy
+        name2serials = self.keyfs.PYPISERIALS.get({})
+        if not name2serials:
+            log.info("retrieving initial name/serial list")
+            name2serials = proxy.list_packages_with_serial()
+            if name2serials is None:
+                from devpi_server.main import fatal
+                fatal("mirror initialization failed: "
+                      "pypi.python.org not reachable")
+            self.keyfs.PYPISERIALS.set(name2serials)
+        else:
+            log.info("reusing already cached name/serial list")
+        self.name2serials = name2serials
+        # create a mapping of normalized name to real name
+        self.normname2name = d = dict()
+        for name in name2serials:
+            norm = normalize_name(name)
+            if norm != name:
+                d[norm] = name
+
+    def _set_project_serial(self, name, serial):
+        """ set the current serial and fill normalization table
+        if project does not exist.
+        """
+        try:
+            current_serial = self.name2serials[name]
+        except KeyError:
+            self.name2serials[name] = serial
+            n = normalize_name(name)
+            if n != name:
+                self.normname2name[n] = name
+        else:
+            self.name2serials[name] = serial
+
     def spawned_pypichanges(self, proxy, proxysleep):
         log.info("changelog/update tasks starting")
         keyfs = self.keyfs
-        name2serials = keyfs.PYPISERIALS.get({})
-        name2serials_version = name2serials.get(".ver", 0)
-        if name2serials_version != self.MIRRORVERSION:
-            log.info("detected MIRRORVERSION CHANGE, restarting caching info")
-            name2serials.clear() # restart getting info from pypi
-                                 # this will not reget release files
-        if name2serials:
-            self.name2serials = name2serials
         while 1:
-            if not name2serials:
-                log.info("retrieving initial name/serial list")
-                init_name2serials = proxy.list_packages_with_serial()
-                if init_name2serials is None:
-                    proxysleep()
-                    continue
-                for name, serial in iteritems(init_name2serials):
-                    name2serials[name.lower()] = serial
-                name2serials[".ver"] = self.MIRRORVERSION
-                keyfs.PYPISERIALS.set(name2serials)
-                self.name2serials = name2serials
-            else:
-                # get changes since the maximum serial we are aware of
-                current_serial = max(itervalues(name2serials))
-                log.debug("querying pypi changelog since %s", current_serial)
-                changelog = proxy.changelog_since_serial(current_serial)
-                if changelog:
-                    names = set()
-                    for x in changelog:
-                        name, version, action, date, serial = x
-                        # XXX remove names if action == "remove"
-                        # and version is None
-                        name = name.lower()
-                        name2serials[name] = max(name2serials.get(name, 0),
-                                                 serial)
-                        names.add(name)
-                    log.debug("got changelog of size %d: %s" %(
-                              len(changelog), names))
-                    keyfs.PYPISERIALS.set(name2serials)
-
-            # walk through all mirrored projects and trigger updates if needed
-            for name in self.getcontained():
-                name = name.lower()
-                serial = name2serials.get(name, 0)
-                self.getreleaselinks(name, refresh=serial)
+            # get changes since the maximum serial we are aware of
+            current_serial = max(itervalues(self.name2serials))
+            log.debug("querying pypi changelog since %s", current_serial)
+            changelog = proxy.changelog_since_serial(current_serial)
+            self.process_changelog(changelog)
+            self.process_refreshes()
             proxysleep()
 
+    def process_changelog(self, changelog):
+        if not changelog:
+            return
+        names = set()
+        for x in changelog:
+            name, version, action, date, serial = x
+            # XXX remove names if action == "remove"
+            # and version is None
+            self.name2serials[name] = max(self.name2serials.get(name, 0),
+                                     serial)
+            names.add(name)
+        log.debug("processed changelog of size %d: %s" %(
+                  len(changelog), names))
+        self.keyfs.PYPISERIALS.set(self.name2serials)
+
+    def process_refreshes(self):
+        # walk through all mirrored projects and trigger updates if needed
+        for name in self.getcontained():
+            serial = self.name2serials.get(name, 0)
+            self.getreleaselinks(name, refresh=serial)
+
 PYPIURL_SIMPLE = "https://pypi.python.org/simple/"
 PYPIURL = "https://pypi.python.org/"
 
