@@ -18,7 +18,7 @@ from .urlutil import DistURL, splitbasename
 from logging import getLogger
 log = getLogger(__name__)
 
-class ReleaseFileStore:
+class FileStore:
     def __init__(self, keyfs):
         self.keyfs = keyfs
 
@@ -33,7 +33,8 @@ class ReleaseFileStore:
                 log.info("replaced md5, deleting stale %s" % entry.relpath)
                 entry.FILE.delete()
             else:
-                log.info("replaced md5 info for %s" % entry.relpath)
+                if entry.md5:
+                    log.info("replaced md5 info for %s" % entry.relpath)
         entry.set(**mapping)
         assert entry.url
         return entry
@@ -112,17 +113,19 @@ class ReleaseFileStore:
             log.info("finished getting remote %r", entry.url)
         return entry.gethttpheaders(), iter_and_cache()
 
-    def store(self, user, index, filename, content):
+    def store(self, user, index, filename, content, last_modified=None):
         md5 = getmd5(content)
-        proj, version = splitbasename(filename, suffix=True)[:2]
+        proj, version = splitbasename(filename)[:2]
         key = self.keyfs.STAGEFILE(user=user, index=index,
                                    #md5=md5,
                                    proj=proj, version=version,
                                    filename=filename)
         entry = self.getentry(key.relpath)
         key.set(content)
+        if last_modified is None:
+            last_modified = http_date()
         entry.set(md5=md5, size=len(content),
-                  last_modified=http_date())
+                  last_modified=last_modified)
         return entry
 
     def add_attachment(self, md5, type, data):
@@ -141,12 +144,14 @@ class ReleaseFileStore:
         return self.keyfs.ATTACHMENT(type=type, md5=md5, num=num).get()
 
     def iter_attachments(self, md5, type):
-        import json
         nums = self.keyfs.ATTACHMENT.listnames("num", type=type, md5=md5)
         for num in range(len(nums)):
             a = self.keyfs.ATTACHMENT(num=str(num), type=type, md5=md5).get()
             yield json.loads(a)
 
+    def iter_attachment_types(self, md5):
+        return self.keyfs.ATTACHMENT.listnames("type", md5=md5, num="0")
+
 
 def getmd5(content):
     md5 = hashlib.md5()
@@ -166,6 +171,9 @@ class RelPathEntry(object):
         #log.debug("self.FILE %s", self.FILE)
         self._mapping = self.PATHENTRY.get()
 
+    def __repr__(self):
+        return "<RelPathEntry %r>" %(self.relpath)
+
     def gethttpheaders(self):
         headers = {}
         if self.last_modified:
