@@ -10,16 +10,56 @@ import threading
 from logging import getLogger
 log = getLogger(__name__)
 
-from devpi_server.types import cached_property
-from devpi_server.config import parseoptions, configure_logging
+from .types import cached_property
+from .config import parseoptions, configure_logging
+from .urlutil import Version
+
 import devpi_server
+from pkg_resources import parse_version
+from . import extpypi
+
+PYPIURL_XMLRPC = "https://pypi.python.org/pypi/"
+
+class Fatal(Exception):
+    pass
+
+def fatal(msg):
+    raise Fatal(msg)
+
+def check_compatible_version(xom):
+    args = xom.config.args
+    if args.upgrade_state or args.export:
+        return
+    server_version = Version(devpi_server.__version__)
+    serverdir = xom.config.serverdir
+    versionfile = serverdir.join(".serverversion")
+    if not versionfile.check():
+        if not xom.db.is_empty():
+            fatal("serverdir %s is unversioned, assuming 1.0.\n"
+                  "Use --upgrade-state to migrate your data. " % serverdir)
+        # pypi mirror state is going to be invalidated anyway.
+    else:
+        state_version = Version(versionfile.read())
+        if server_version < state_version:
+            fatal("Incompatible state: server %s cannot run serverdir "
+                  "%s in %s. " %(server_version, serverdir, state_version))
+    versionfile.write(server_version.string)
+    log.info("setting new server version %s in serverdir" %(server_version))
 
 def main(argv=None):
     """ devpi-server command line entry point. """
+    try:
+        return _main(argv)
+    except Fatal as e:
+        tw = py.io.TerminalWriter(sys.stderr)
+        tw.line("fatal: %s" %  e.args[0], red=True)
+        return 1
+
+def _main(argv=None):
     if argv is None:
         argv = sys.argv
-    argv = map(str, argv)
 
+    argv = map(str, argv)
     config = parseoptions(argv)
     args = config.args
 
@@ -27,10 +67,13 @@ def main(argv=None):
         print (devpi_server.__version__)
         return
 
+    # meta commmands (that will not instantiate server object in-process)
     if args.gendeploy:
         from devpi_server.gendeploy import gendeploy
         return gendeploy(config)
 
+    xom = XOM(config)
+    check_compatible_version(xom)
     if args.start or args.stop or args.log or args.status:
         xprocdir = config.serverdir.join(".xproc")
         from devpi_server.bgserver import BackgroundServer
@@ -50,12 +93,12 @@ def main(argv=None):
                 bgserver.line("no server is running")
             return
 
-    configure_logging(config)
-    xom = XOM(config)
     if args.passwd:
         from devpi_server.db import run_passwd
         return run_passwd(xom.db, config.args.passwd)
-    return bottle_run(xom)
+
+    return xom.main()
+
 
 def make_application():
     """ entry point for making an application object with defaults. """
@@ -72,7 +115,7 @@ def bottle_run(xom):
     hostaddr = "http://%s:%s" %(xom.config.args.host, xom.config.args.port)
     log.info("serving at url: %s", hostaddr)
     log.info("bug tracker: https://bitbucket.org/hpk42/devpi/issues")
-    log.info("IRC: #pylib on irc.freenode.net")
+    log.info("IRC: #devpi on irc.freenode.net")
     if "WEBTRACE" in os.environ and xom.config.args.debug:
         from weberror.evalexception import make_eval_exception
         app = make_eval_exception(app, {})
@@ -100,7 +143,7 @@ def add_keys(keyfs):
     keyfs.USER = keyfs.addkey("{user}/.config", dict)
 
     # type pypimirror related data
-    keyfs.PYPILINKS = keyfs.addkey("root/pypi/links/{name}", list)
+    keyfs.PYPILINKS = keyfs.addkey("root/pypi/links/{name}", dict)
     keyfs.PYPIFILES = keyfs.addkey("root/pypi/f/{relpath}", file)
     keyfs.PYPISERIALS = keyfs.addkey("root/pypi/serials", dict)
 
@@ -121,11 +164,38 @@ class XOM:
     class Exiting(SystemExit):
         pass
 
-    def __init__(self, config):
+    def __init__(self, config, proxy=None, httpget=None):
         self.config = config
         self._spawned = []
         self._shutdown = threading.Event()
         self._shutdownfuncs = []
+        if proxy is not None:
+            self.proxy = proxy
+
+        if httpget is not None:
+            self.httpget = httpget
+
+    def main(self):
+        xom = self
+        args = xom.config.args
+        if args.upgrade_state:
+            from devpi_server.importexport import do_upgrade
+            return do_upgrade(xom)
+
+        if args.export:
+            from devpi_server.importexport import do_export
+            return do_export(args.export, xom)
+        configure_logging(xom.config)
+        # access extdb to make sure invalidation happens
+        xom.extdb
+        if args.import_:
+            from devpi_server.importexport import do_import
+            return do_import(args.import_, xom)
+        return bottle_run(xom)
+
+    def fatal(self, msg):
+        self.shutdown()
+        fatal(msg)
 
     def shutdown(self):
         log.debug("shutting down")
@@ -162,9 +232,9 @@ class XOM:
         return thread
 
     @cached_property
-    def releasefilestore(self):
-        from devpi_server.filestore import ReleaseFileStore
-        return ReleaseFileStore(self.keyfs)
+    def filestore(self):
+        from devpi_server.filestore import FileStore
+        return FileStore(self.keyfs)
 
     @cached_property
     def keyfs(self):
@@ -176,7 +246,16 @@ class XOM:
     @cached_property
     def extdb(self):
         from devpi_server.extpypi import ExtDB
-        return ExtDB(xom=self)
+        extdb = ExtDB(keyfs=self.keyfs, httpget=self.httpget,
+                      filestore=self.filestore,
+                      proxy=self.proxy)
+        return extdb
+
+    @cached_property
+    def proxy(self):
+        from devpi_server.extpypi import XMLProxy
+        from xmlrpclib import ServerProxy
+        return XMLProxy(ServerProxy(PYPIURL_XMLRPC))
 
     @cached_property
     def db(self):
@@ -214,15 +293,17 @@ class XOM:
     def create_app(self, catchall=True, immediatetasks=False):
         from devpi_server.views import PyPIView, route
         from bottle import Bottle
-        log.info("creating application in process %s", os.getpid())
+        log.debug("creating application in process %s", os.getpid())
         app = Bottle(catchall=catchall)
-        plugin = BackgroundPlugin(xom=self)
         if immediatetasks == -1:
             pass
-        elif immediatetasks:
-            plugin.start_background_tasks()
-        else:  # defer to when the first request arrives
-            app.install(plugin)
+        else:
+            plugin = BackgroundPlugin(xom=self)
+            if immediatetasks:
+                plugin.start_background_tasks()
+            else:  # defer to when the first request arrives
+                app.install(plugin)
+        app.xom = self
         pypiview = PyPIView(self)
         route.discover_and_call(pypiview, app.route)
         return app
@@ -235,6 +316,7 @@ class BackgroundPlugin:
 
     def __init__(self, xom):
         self.xom = xom
+        assert xom.proxy
 
     def setup(self, app):
         log.debug("plugin.setup(%r)", app)
@@ -253,11 +335,7 @@ class BackgroundPlugin:
         log.debug("plugin.close() called")
 
     def start_background_tasks(self):
-        from devpi_server.extpypi import XMLProxy
-        from xmlrpclib import ServerProxy
         xom = self.xom
-        PYPIURL_XMLRPC = "https://pypi.python.org/pypi/"
-        xom.proxy = XMLProxy(ServerProxy(PYPIURL_XMLRPC))
         self._thread = xom.spawn(xom.extdb.spawned_pypichanges,
             args=(xom.proxy, lambda: xom.sleep(xom.config.args.refresh)))
 
@@ -270,7 +348,8 @@ class FatalResponse:
 
 def set_default_indexes(db):
     PYPI = "root/pypi"
-    if "root" not in db.user_list():
-        db.user_create("root", password="")
     if not db.index_exists(PYPI):
+        if "root" not in db.user_list():
+            db.user_create("root", password="")
         db.index_create(PYPI, bases=(), type="mirror", volatile=False)
+        print ("set root/pypi default index")
