@@ -1,18 +1,19 @@
-
 import py
 from py.xml import html
 from devpi_server.types import lazydecorator, cached_property
-from bottle import response, request, abort, redirect, HTTPError, auth_basic
+import devpi_server
+from bottle import response, request, abort, redirect, HTTPError
 from bottle import BaseResponse, HTTPResponse, static_file
-from devpi_server.config import render_string
-from devpi_server import urlutil
 import bottle
 import json
-import itsdangerous
 import logging
 import inspect
-
 import requests
+from .validation import normalize_name, is_valid_archive_name
+
+from .auth import Auth
+from .config import render_string
+from . import urlutil
 
 log = logging.getLogger(__name__)
 
@@ -38,10 +39,23 @@ def simple_html_body(title, bodytags, extrahead=""):
 #                       status=code, headers=
 #                       {"content-type": "application/json"})
 
+API_VERSION = "1"
+
+meta_headers = {"X-DEVPI-API-VERSION": API_VERSION,
+                "X-DEVPI-SERVER-VERSION": devpi_server.__version__}
+
 def abort(code, body):
     if "application/json" in request.headers.get("Accept", ""):
         apireturn(code, body)
-    bottle.abort(code, body)
+    error = HTTPError(code, body)
+    for n,v in meta_headers.items():
+        error.add_header(n, v)
+    raise error
+
+def abort_custom(code, msg):
+    error = HTTPError(code, msg)
+    error.status = "%s %s" %(code, msg)
+    raise error
 
 def abort_authenticate(msg="authentication required"):
     err = HTTPError(401, msg)
@@ -58,8 +72,9 @@ def apireturn(code, message=None, result=None, type=None):
     if message:
         d["message"] = message
     data = json.dumps(d, indent=2) + "\n"
-    raise HTTPResponse(body=data, status=code, header=
-                    {"content-type": "application/json"})
+    header = meta_headers.copy()
+    header["content-type"] = "application/json"
+    raise HTTPResponse(body=data, status=code, header=header)
 
 def json_preferred():
     # XXX do proper "best" matching
@@ -67,78 +82,12 @@ def json_preferred():
 
 route = lazydecorator()
 
-class Auth:
-    LOGIN_EXPIRATION = 60*60*10  # 10 hours
-
-    class Expired(Exception):
-        """ proxy authentication expired. """
-
-    def __init__(self, db, secret):
-        self.db = db
-        self.signer = itsdangerous.TimestampSigner(secret)
-
-    def get_auth_user(self, auth, raising=True):
-        try:
-            authuser, authpassword = auth
-        except TypeError:
-            return None
-        try:
-            val = self.signer.unsign(authpassword, self.LOGIN_EXPIRATION)
-        except itsdangerous.SignatureExpired:
-            if raising:
-                raise self.Expired()
-            return None
-        except itsdangerous.BadData:
-            # check if we got user/password direct authentication
-            if self.db.user_validate(authuser, authpassword):
-                return authuser
-            return None
-        else:
-            if not val.startswith(authuser + "-"):
-                log.debug("mismatch credential for user %r", authuser)
-                return None
-            return authuser
-
-    def new_proxy_auth(self, user, password):
-        hash = self.db.user_validate(user, password)
-        if hash:
-            pseudopass = self.signer.sign(user + "-" + hash)
-            return {"password":  pseudopass,
-                    "expiration": self.LOGIN_EXPIRATION}
-
-    def require_user(self, user, stage=None, acltype="upload"):
-        #log.debug("headers %r", request.headers.items())
-        if not self.db.user_exists(user):
-            abort(404, "user %r does not exist" % user)
-        try:
-            auth_user = self.get_auth_user(request.auth)
-        except itsdangerous.SignatureExpired:
-            abort_authenticate(msg="authentication expired")
-
-        if not auth_user:
-            log.warn("invalid or no authentication")
-            log.warn("request.auth %s" %(request.auth,))
-            abort_authenticate()
-        if auth_user == "root" or auth_user == user:
-            return
-        if stage:
-            acl = stage.ixconfig.get("acl_" + acltype, [])
-            if auth_user in acl:
-                log.debug("user %r is acl_upload list", auth_user)
-                return
-            apireturn(403, message="user %r not authorized for %s to %s"
-                             % (auth_user, acltype, stage.name))
-        log.info("user %r not authorized", auth_user)
-        abort_authenticate()
-
-
 class PyPIView:
     def __init__(self, xom):
         self.xom = xom
         self.db = xom.db
         self.auth = Auth(xom.db, xom.config.secret)
 
-
     def getstage(self, user, index):
         stage = self.db.getstage(user, index)
         if not stage:
@@ -156,6 +105,7 @@ class PyPIView:
         api = {
             "resultlog": "/+tests",
             "login": "/+login",
+            "authstatus": self.auth.get_auth_status(request.auth),
         }
         if path:
             parts = path.split("/")
@@ -179,7 +129,7 @@ class PyPIView:
 
     @route("/+tests", method="POST")
     def add_attach(self):
-        filestore = self.xom.releasefilestore
+        filestore = self.xom.filestore
         data = getjson(request)
         md5 = data["installpkg"]["md5"]
         num = filestore.add_attachment(md5=md5, type="toxresult",
@@ -189,13 +139,13 @@ class PyPIView:
 
     @route("/+tests/<md5>/<type>", method="GET")
     def get_attachlist(self, md5, type):
-        filestore = self.xom.releasefilestore
+        filestore = self.xom.filestore
         datalist = list(filestore.iter_attachments(md5=md5, type=type))
         apireturn(200, type="list:toxresult", result=datalist)
 
     @route("/+tests/<md5>/<type>/<num>", method="GET")
     def get_attach(self, md5, type, num):
-        filestore = self.xom.releasefilestore
+        filestore = self.xom.filestore
         data = filestore.get_attachment(md5=md5, type=type, num=num)
         apireturn(200, type="testresult", result=data)
 
@@ -210,11 +160,18 @@ class PyPIView:
     @route("/<user>/<index>/+simple/<projectname>/")
     def simple_list_project(self, user, index, projectname):
         # we only serve absolute links so we don't care about the route's slash
+        abort_if_invalid_projectname(projectname)
         stage = self.getstage(user, index)
+        info = stage.get_project_info(projectname)
+        if info and info.name != projectname:
+            redirect("/%s/+simple/%s/" % (stage.name, info.name))
         result = stage.getreleaselinks(projectname)
         if isinstance(result, int):
             if result == 404:
-                abort(404, "no such project")
+                # we don't want pip/easy_install to try the whole simple
+                # page -- we know for sure there is no fitting project
+                # because all devpi indexes perform package name normalization
+                abort(200, "no such project %r" % projectname)
             if result >= 500:
                 abort(502, "upstream server has internal error")
             if result < 0:
@@ -269,7 +226,7 @@ class PyPIView:
 
     @route("/<user>/<index>", method=["PUT", "PATCH"])
     def index_create_or_modify(self, user, index):
-        self.auth.require_user(user)
+        self.require_user(user)
         ixconfig = self.db.index_get(user, index)
         if request.method == "PUT" and ixconfig is not None:
             apireturn(409, "index %s/%s exists" % (user, index))
@@ -290,7 +247,7 @@ class PyPIView:
 
     @route("/<user>/<index>", method=["DELETE"])
     def index_delete(self, user, index):
-        self.auth.require_user(user)
+        self.require_user(user)
         indexname = user + "/" + index
         ixconfig = self.db.index_get(user, index)
         if not ixconfig:
@@ -321,19 +278,27 @@ class PyPIView:
         except KeyError:
             apireturn(400, message="no name/version specified in json")
 
-        metadata = stage.get_metadata(name, version)
-        entries = stage.getreleaselinks_perstage(name)
+        projectconfig = stage.get_projectconfig(name)
         matches = []
-        for entry in entries:
-            n, v = urlutil.guess_pkgname_and_version(entry.basename)
-            if n == name and str(v) == version:
-                matches.append(entry)
-        if not matches or not metadata:
+        if projectconfig:
+            verdata = projectconfig.get(version)
+            if verdata:
+                files = verdata.get("+files")
+                for basename, relpath in files.items():
+                    entry = stage.xom.filestore.getentry(relpath)
+                    if not entry.iscached():
+                        abort(400, "cannot push non-cached files")
+                    matches.append(entry)
+                metadata = get_pure_metadata(verdata)
+
+        if not matches:
             log.info("%s: no release files %s-%s" %(stage.name, name, version))
             apireturn(404,
                       message="no release/files found for %s-%s" %(
                       name, version))
 
+        doczip = stage.get_doczip(name)
+
         # prepare metadata for submission
         metadata[":action"] = "submit"
 
@@ -363,6 +328,10 @@ class PyPIView:
                     res = 200
                 results.append((res, "store_releasefile", entry.basename,
                                 "->", target_stage.name))
+            if doczip:
+                target_stage.store_doczip(name, doczip)
+                results.append((200, "uploaded documentation", name,
+                                "->", target_stage.name))
             apireturn(200, result=results, type="actionlog")
         else:
             posturl = pushdata["posturl"]
@@ -376,17 +345,28 @@ class PyPIView:
             results.append((r.status_code, "register", name, version))
             if r.status_code in ok_codes:
                 for entry in matches:
-                    metadata[":action"] = "file_upload"
+                    file_metadata = metadata.copy()
+                    file_metadata[":action"] = "file_upload"
                     basename = entry.basename
                     pyver, filetype = urlutil.get_pyversion_filetype(basename)
-                    metadata["filetype"] = filetype
-                    metadata["pyversion"] = pyver
+                    file_metadata["filetype"] = filetype
+                    file_metadata["pyversion"] = pyver
                     openfile = entry.FILE.filepath.open("rb")
                     log.info("sending %s to %s", basename, posturl)
-                    r = requests.post(posturl, data=metadata, auth=pypiauth,
+                    r = requests.post(posturl, data=file_metadata,
+                          auth=pypiauth,
                           files={"content": (basename, openfile)})
                     log.debug("send finished, status: %s", r.status_code)
                     results.append((r.status_code, "upload", entry.relpath))
+                if doczip:
+                    doc_metadata = metadata.copy()
+                    doc_metadata[":action"] = "doc_upload"
+                    r = requests.post(posturl, data=doc_metadata,
+                          auth=pypiauth,
+                          files={"content": (name + ".zip", doczip)})
+                    log.debug("send finished, status: %s", r.status_code)
+                    results.append((r.status_code, "docfile", name))
+                #
             if r.status_code in ok_codes:
                 apireturn(200, result=results, type="actionlog")
             else:
@@ -397,7 +377,7 @@ class PyPIView:
         if user == "root" and index == "pypi":
             abort(404, "cannot submit to pypi mirror")
         stage = self.getstage(user, index)
-        self.auth.require_user(user, stage=stage)
+        self.require_user(user, stage=stage)
         try:
             action = request.forms[":action"]
         except KeyError:
@@ -410,16 +390,27 @@ class PyPIView:
             except KeyError:
                 abort(400, "content file field not found")
             name = request.forms.get("name")
-            version = request.forms.get("version", "")
-            if not stage.get_metadata(name, version):
-                self._register_metadata(stage, request.forms)
+            version = request.forms.get("version")
+            info = stage.get_project_info(name)
+            if not info:
+                abort(400, "no project named %r was ever registered" %(name))
             if action == "file_upload":
+                abort_if_invalid_filename(name, content.filename)
+                if not stage.get_metadata(name, version):
+                    self._register_metadata(stage, request.forms)
                 res = stage.store_releasefile(content.filename, content.value)
                 if res == 409:
                     abort(409, "%s already exists in non-volatile index" %(
                          content.filename,))
-                trigger_jenkins(stage, name)
+                jenkins_url = stage.ixconfig["uploadtrigger_jenkins"]
+                if jenkins_url:
+                    jenkins_url = jenkins_url.format(pkgname=name)
+                    if trigger_jenkins(stage, jenkins_url, name) == -1:
+                        abort_custom(200,
+                            "OK, but couldn't trigger jenkins at %s" %
+                            (jenkins_url,))
             else:
+                # docs have no version (XXX but they are tied to the latest)
                 if len(content.value) > MAXDOCZIPSIZE:
                     abort(413, "zipfile too large")
                 stage.store_doczip(name, content.value)
@@ -431,7 +422,15 @@ class PyPIView:
         metadata = {}
         for key in stage.metadata_keys:
             metadata[key] = form.get(key, "")
-        stage.register_metadata(metadata)
+        try:
+            stage.register_metadata(metadata)
+        except stage.RegisterNameConflict as e:
+            info = e.args[0]
+            abort_custom(403, "cannot register %r because %r is already "
+                  "registered at %s" % (
+                  metadata["name"], info.name, info.stage.name))
+        except ValueError as e:
+            abort_custom(400, "invalid metadata: %s" % (e,))
         log.info("%s: got submit release info %r",
                  stage.name, metadata["name"])
 
@@ -454,6 +453,9 @@ class PyPIView:
     @route("/<user>/<index>/<name>/")
     def project_get(self, user, index, name):
         stage = self.getstage(user, index)
+        info = stage.get_project_info(name)
+        if info and info.name != name:
+            redirect("/%s/%s/" % (stage.name, info.name))
         metadata = stage.get_projectconfig(name)
         #if not metadata:
         #    apireturn("404", "project %r does not exist" % name)
@@ -469,7 +471,7 @@ class PyPIView:
 
     @route("/<user>/<index>/<name>", method="PUT")
     def project_add(self, user, index, name):
-        self.auth.require_user(user)
+        self.require_user(user)
         stage = self.getstage(user, index)
         if stage.project_exists(name):
             apireturn(409, "project %r exists" % name)
@@ -479,7 +481,7 @@ class PyPIView:
     @route("/<user>/<index>/<name>", method="DELETE")
     @route("/<user>/<index>/<name>/", method="DELETE")
     def project_delete(self, user, index, name):
-        self.auth.require_user(user)
+        self.require_user(user)
         stage = self.getstage(user, index)
         if stage.name == "root/pypi":
             abort(405, "cannot delete on root/pypi index")
@@ -547,7 +549,7 @@ class PyPIView:
         relpath = request.path.strip("/")
         if "#" in relpath:   # XXX unclear how this can happen (it does)
             relpath = relpath.split("#", 1)[0]
-        filestore = self.xom.releasefilestore
+        filestore = self.xom.filestore
         if json_preferred():
             entry = filestore.getentry(relpath)
             if not entry.exists():
@@ -570,6 +572,13 @@ class PyPIView:
             projectlist = stage.getprojectnames_perstage()
             projectlist = sorted(projectlist)
             apireturn(200, type="list:projectconfig", result=projectlist)
+        if stage.name == "root/pypi":
+            return simple_html_body("%s index" % stage.name, [
+                html.ul(
+                    html.li(html.a("simple index", href="+simple/")),
+                ),
+            ]).unicode()
+
 
         # XXX this should go to a template
         if hasattr(stage, "ixconfig"):
@@ -586,33 +595,25 @@ class PyPIView:
         else:
             bases = []
         latest_packages = html.ul()
-        for name in stage.getprojectnames_perstage():
-            for entry in stage.getreleaselinks(name):
-                if entry.eggfragment:
-                    continue
-                if not entry.relpath.startswith(stage.name + "/"):
-                    break
-                if entry.url:
-                    path = entry.url
-                else:
-                    path = entry.relpath
-                name, ver = urlutil.DistURL(path).pkgname_and_version
-                dockey = self.db.keyfs.STAGEDOCS(user=user,
-                                                 index=index, name=name)
-                if dockey.exists():
-                    docs = [" docs: ", html.a("%s-%s docs" %(name, ver),
+        for projectname in stage.getprojectnames_perstage():
+            metadata = stage.get_metadata_latest(projectname)
+            name, ver = metadata["name"], metadata["version"]
+            dockey = stage._doc_key(name)
+            if dockey.exists():
+                docs = [" docs: ", html.a("%s-%s docs" %(name, ver),
                                 href="%s/+doc/index.html" %(name))]
-                else:
-                    docs = []
-
+            else:
+                docs = []
+            for basename, relpath in metadata["+files"].items():
                 latest_packages.append(html.li(
                     html.a("%s-%s info page" % (name, ver),
                            href="%s/%s/" % (name, ver)),
                     " releasefiles: ",
-                    html.a(entry.basename, href="/" + entry.relpath),
+                    html.a(basename, href="/" + relpath),
                     *docs
                 ))
-                break
+                break  # could present more releasefiles
+
         latest_packages = [
             html.h2("in-stage latest packages, at least as recent as bases"),
             latest_packages]
@@ -629,6 +630,32 @@ class PyPIView:
     #
     # login and user handling
     #
+
+    def require_user(self, user, stage=None, acltype="upload"):
+        #log.debug("headers %r", request.headers.items())
+        status, auth_user = self.auth.get_auth_status(request.auth)
+        log.debug("got auth status %r for user %r" %(status, auth_user))
+        if not self.db.user_exists(user):
+            abort(404, "required user %r does not exist" % auth_user)
+        if status == "nouser":
+            abort(404, "user %r does not exist" % auth_user)
+        elif status == "expired":
+            abort_authenticate(msg="auth expired for %r" % auth_user)
+        elif status == "noauth":
+            abort_authenticate()
+        if auth_user == "root" or auth_user == user:
+            return
+        if stage:
+            acl = stage.ixconfig.get("acl_" + acltype, [])
+            if auth_user in acl:
+                log.debug("user %r is acl_upload list", auth_user)
+                return
+            apireturn(403, message="user %r not authorized for %s to %s"
+                             % (auth_user, acltype, stage.name))
+        log.info("user %r not authorized", auth_user)
+        abort_authenticate()
+
+
     @route("/+login", method="POST")
     def login(self):
         dict = getjson(request)
@@ -639,13 +666,14 @@ class PyPIView:
             abort(400, "Bad request: no user/password specified")
         proxyauth = self.auth.new_proxy_auth(user, password)
         if proxyauth:
-            return proxyauth
+            apireturn(200, "login successful", type="proxyauth",
+                result=proxyauth)
         apireturn(401, "user %r could not be authenticated" % user)
 
     @route("/<user>", method="PATCH")
     @route("/<user>/", method="PATCH")
     def user_patch(self, user):
-        self.auth.require_user(user)
+        self.require_user(user)
         dict = getjson(request, allowed_keys=["email", "password"])
         email = dict.get("email")
         password = dict.get("password")
@@ -669,7 +697,7 @@ class PyPIView:
     def user_delete(self, user):
         if user == "root":
             apireturn(403, "root user cannot be deleted")
-        self.auth.require_user(user)
+        self.require_user(user)
         userconfig = self.db.user_get(user)
         if not userconfig:
             apireturn(404, "user %r does not exist" % user)
@@ -682,7 +710,7 @@ class PyPIView:
 
     @route("/<user>", method="GET")
     def user_get(self, user):
-        #self.auth.require_user(user)
+        #self.require_user(user)
         userconfig = self.db.user_get(user)
         if not userconfig:
             apireturn(404, "user %r does not exist" % user)
@@ -720,11 +748,7 @@ def get_outside_url(headers, outsideurl):
     log.debug("host header: %s", url)
     return url
 
-def trigger_jenkins(stage, testspec):
-    jenkins_url = stage.ixconfig["uploadtrigger_jenkins"]
-    if not jenkins_url:
-        return
-    jenkins_url = jenkins_url.format(pkgname=testspec)
+def trigger_jenkins(stage, jenkins_url, testspec):
     baseurl = get_outside_url(request.headers,
                               stage.xom.config.args.outside_url)
 
@@ -737,19 +761,39 @@ def trigger_jenkins(stage, testspec):
         DEVPI_INSTALL_INDEX = baseurl + stage.name + "/+simple/"
     )
     inputfile = py.io.BytesIO(source)
-    r = requests.post(jenkins_url, data={
-                    "Submit": "Build",
-                    "name": "jobscript.py",
-                    "json": json.dumps(
-                {"parameter": {"name": "jobscript.py", "file": "file0"}}),
-        },
-            files={"file0": ("file0", inputfile)})
+    try:
+        r = requests.post(jenkins_url, data={
+                        "Submit": "Build",
+                        "name": "jobscript.py",
+                        "json": json.dumps(
+                    {"parameter": {"name": "jobscript.py", "file": "file0"}}),
+            },
+                files={"file0": ("file0", inputfile)})
+    except requests.exceptions.RequestException as e:
+        log.error("%s: failed to connect to jenkins at %s",
+                  testspec, jenkins_url)
+        return -1
 
-    if r.status_code == 200:
+    if 200 <= r.status_code < 300:
         log.info("successfully triggered jenkins: %s", jenkins_url)
     else:
         log.error("%s: failed to trigger jenkins at %s", r.status_code,
                   jenkins_url)
+        return -1
+
+def abort_if_invalid_filename(name, filename):
+    if not is_valid_archive_name(filename):
+        abort_custom(400, "%r is not a valid archive name" %(filename))
+    if normalize_name(filename).startswith(normalize_name(name)):
+        return
+    abort_custom(400, "filename %r does not match project name %r"
+                      %(filename, name))
+
+def abort_if_invalid_projectname(projectname):
+    try:
+        projectname.decode("ascii")
+    except UnicodeDecodeError:
+        abort(400, "unicode project names not allowed")
 
 
 def getkvdict_index(req):
@@ -772,3 +816,10 @@ def getkvdict_index(req):
     if "uploadtrigger_jenkins" in req:
         kvdict["uploadtrigger_jenkins"] = req["uploadtrigger_jenkins"]
     return kvdict
+
+def get_pure_metadata(somedict):
+    metadata = {}
+    for n, v in somedict.items():
+        if n[0] != "+":
+            metadata[n] = v
+    return metadata
