@@ -3,9 +3,12 @@ import re
 import logging
 from webtest.forms import Upload
 import mimetypes
+import mock
 import pytest
 import py
-from devpi_server.main import XOM, add_keys
+from devpi_server.main import XOM, parseoptions
+from devpi_server.extpypi import XMLProxy
+from devpi_server.urlutil import splitbasename
 
 
 log = logging.getLogger(__name__)
@@ -33,27 +36,68 @@ def caplog(caplog):
     return caplog
 
 @pytest.fixture
-def xom_notmocked(request):
-    from devpi_server.main import parseoptions, XOM
-    config = parseoptions(["devpi-server"])
-    xom = XOM(config)
-    request.addfinalizer(xom.shutdown)
-    return xom
+def gentmp(request):
+    tmpdirhandler = request.config._tmpdirhandler
+    cache = []
+    def gentmp(name=None):
+        if not cache:
+            prefix = re.sub("[\W]", "_", request.node.name)
+            basedir = tmpdirhandler.mktemp(prefix, numbered=True)
+            cache.append(basedir)
+        else:
+            basedir = cache[0]
+        if name:
+            return basedir.mkdir(name)
+        return py.path.local.make_numbered_dir(prefix="gentmp",
+                keep=0, rootdir=basedir, lock_timeout=None)
+    return gentmp
 
 @pytest.fixture
-def xom(request, keyfs, filestore, httpget):
-    from devpi_server.main import parseoptions, XOM
-    from devpi_server.extpypi import ExtDB
-    config = parseoptions(["devpi-server"])
-    xom = XOM(config)
-    xom.keyfs = keyfs
-    xom.releasefilestore = filestore
-    xom.httpget = httpget
-    xom.extdb = ExtDB(xom=xom)
-    xom.extdb.setextsimple = httpget.setextsimple
-    xom.extdb.url2response = httpget.url2response
-    request.addfinalizer(xom.shutdown)
-    return xom
+def xom_notmocked(request, makexom):
+    return makexom(mocking=False)
+
+@pytest.fixture
+def xom(request, makexom):
+    return makexom([])
+
+@pytest.fixture
+def makexom(request, gentmp, httpget):
+    def makexom(opts=(), httpget=httpget, proxy=None, mocking=True):
+        serverdir = gentmp()
+        fullopts = ["devpi-server", "--serverdir", serverdir] + list(opts)
+        fullopts = [str(x) for x in fullopts]
+        config = parseoptions(fullopts)
+        if mocking:
+            if proxy is None:
+                proxy = mock.create_autospec(XMLProxy)
+                proxy.list_packages_with_serial.return_value = {}
+            xom = XOM(config, proxy=proxy, httpget=httpget)
+            add_extdb_mocks(xom.extdb, httpget)
+        else:
+            xom = XOM(config)
+        request.addfinalizer(xom.shutdown)
+        return xom
+    return makexom
+
+
+@pytest.fixture
+def maketestapp(request):
+    def maketestapp(xom):
+        app = xom.create_app(catchall=False, immediatetasks=-1)
+        mt = MyTestApp(app)
+        mt.xom = xom
+        return mt
+    return maketestapp
+
+@pytest.fixture
+def makemapp(request, maketestapp, makexom):
+    def makemapp(testapp=None, options=()):
+        if testapp is None:
+            testapp = maketestapp(makexom(options))
+        m = Mapp(testapp)
+        m.xom = testapp.xom
+        return m
+    return makemapp
 
 @pytest.fixture
 def httpget(pypiurls):
@@ -69,7 +113,8 @@ def httpget(pypiurls):
                     if fakeresponse is None:
                         fakeresponse = dict(status_code = 404)
                     xself.__dict__.update(fakeresponse)
-                    xself.url = url
+                    if "url" not in fakeresponse:
+                        xself.url = url
                     xself.allow_redirects = allow_redirects
                 def __repr__(xself):
                     return "<mockresponse %s url=%s>" % (xself.status_code,
@@ -78,11 +123,11 @@ def httpget(pypiurls):
             log.debug("returning %s", r)
             return r
 
-        def mockresponse(self, url, **kw):
+        def mockresponse(self, mockurl, **kw):
             if "status_code" not in kw:
                 kw["status_code"] = 200
-            log.debug("set mocking response %s %s", url, kw)
-            self.url2response[url] = kw
+            log.debug("set mocking response %s %s", mockurl, kw)
+            self.url2response[mockurl] = kw
 
         def setextsimple(self, name, text=None, pypiserial=10000, **kw):
             headers = kw.setdefault("headers", {})
@@ -105,21 +150,27 @@ def httpget(pypiurls):
     return MockHTTPGet()
 
 @pytest.fixture
-def filestore(keyfs):
-    from devpi_server.filestore import ReleaseFileStore
-    return ReleaseFileStore(keyfs)
+def filestore(xom):
+    return xom.filestore
 
 @pytest.fixture
-def keyfs(tmpdir):
-    from devpi_server.keyfs import KeyFS
-    keyfs = KeyFS(tmpdir.join("keyfs"))
-    add_keys(keyfs)
-    return keyfs
+def keyfs(xom):
+    return xom.keyfs
 
 @pytest.fixture
 def extdb(xom):
     return xom.extdb
 
+def add_extdb_mocks(extdb, httpget):
+    # add some mocking helpers
+    extdb.url2response = httpget.url2response
+    def setextsimple(name, text=None, pypiserial=10000, **kw):
+        extdb._set_project_serial(name, pypiserial)
+        httpget.setextsimple(name, text=text, pypiserial=pypiserial, **kw)
+    extdb.setextsimple = setextsimple
+    extdb.mock_simple = setextsimple
+    extdb.httpget = httpget
+
 @pytest.fixture
 def pypiurls():
     from devpi_server.extpypi import PYPIURL_SIMPLE, PYPIURL
@@ -138,30 +189,28 @@ def db(xom):
     return db
 
 @pytest.fixture
-def mapp(testapp):
-    return Mapp(testapp)
+def mapp(makemapp, testapp):
+    return makemapp(testapp)
 
-class Mapp:
+
+from .functional import MappMixin
+
+
+class Mapp(MappMixin):
     def __init__(self, testapp):
         self.testapp = testapp
+        self.current_stage = ""
+
+    def _getindexname(self, indexname):
+        if not indexname:
+            assert self.current_stage, "no index in use, none specified"
+            return self.current_stage
+        return indexname
 
     def delete_user(self, user, code=200):
         r = self.testapp.delete_json("/%s" % user, expect_errors=True)
         assert r.status_code == code
 
-    def getapi(self, relpath="/"):
-        path = relpath.strip("/")
-        if not path:
-            path = "/+api"
-        else:
-            path = "/%s/+api" % path
-        r = self.testapp.get(path)
-        assert r.status_code == 200
-        class API:
-            def __init__(self):
-                self.__dict__.update(r.json["result"])
-        return API()
-
     def login(self, user="root", password="", code=200):
         api = self.getapi()
         r = self.testapp.post_json(api.login,
@@ -169,8 +218,9 @@ class Mapp:
                                   expect_errors=True)
         assert r.status_code == code
         if code == 200:
-            self.testapp.set_auth(user, r.json["password"])
-            self.auth = user, r.json["password"]
+            password = r.json.get("result", r.json)["password"]
+            self.testapp.set_auth(user, password)
+            self.auth = user, password
 
     def login_root(self):
         self.login("root", "")
@@ -228,12 +278,22 @@ class Mapp:
         self.create_user(user, password)
         self.login(user, password)
 
+    def use(self, stagename):
+        stagename = stagename.strip("/")
+        assert stagename.count("/") == 1, stagename
+        self.api = self.getapi(stagename)
+        self.api.stagename = stagename
+        self.current_stage = stagename
+        return self.api
+
     def getjson(self, path, code=200):
         r = self.testapp.get_json(path, {}, expect_errors=True)
         assert r.status_code == code
+        if r.status_code == 302:
+            return r.headers["location"]
         return r.json
 
-    def create_index(self, indexname, indexconfig=None, code=200):
+    def create_index(self, indexname, indexconfig=None, use=True, code=200):
         if indexconfig is None:
             indexconfig = {}
         if "/" in indexname:
@@ -246,6 +306,7 @@ class Mapp:
         assert r.status_code == code
         if code in (200,201):
             assert r.json["result"]["type"] == "stage"
+            return self.use("%s/%s" % (user, index))
         if code in (400,):
             return r.json["message"]
 
@@ -259,7 +320,8 @@ class Mapp:
                                      expect_errors=True)
         assert r.status_code == code
 
-    def set_uploadtrigger_jenkins(self, indexname, triggerurl):
+    def set_uploadtrigger_jenkins(self, triggerurl, indexname=None):
+        indexname = self._getindexname(indexname)
         indexurl = "/" + indexname
         r = self.testapp.get_json(indexurl)
         result = r.json["result"]
@@ -267,7 +329,8 @@ class Mapp:
         r = self.testapp.patch_json(indexurl, result)
         assert r.status_code == 200
 
-    def set_acl(self, indexname, users, acltype="upload"):
+    def set_acl(self, users, acltype="upload", indexname=None):
+        indexname = self._getindexname(indexname)
         r = self.testapp.get_json("/%s" % indexname)
         result = r.json["result"]
         if not isinstance(users, list):
@@ -277,39 +340,69 @@ class Mapp:
         r = self.testapp.patch_json("/%s" % (indexname,), result)
         assert r.status_code == 200
 
-    def get_acl(self, indexname, acltype="upload"):
+    def get_acl(self, acltype="upload", indexname=None):
+        indexname = self._getindexname(indexname)
         r = self.testapp.get_json("/%s" % indexname)
         return r.json["result"].get("acl_" + acltype, None)
 
-    def create_project(self, indexname, projectname, code=201):
+    def create_project(self, projectname, code=201, indexname=None):
+        indexname = self._getindexname(indexname)
         user, password = self.testapp.auth
-        r = self.testapp.put_json("/%s/%s/%s" % (user, indexname,
+        r = self.testapp.put_json("/%s/%s" % (indexname,
                                   projectname), {}, expect_errors=True)
         assert r.status_code == code
         if code == 201:
             assert "created" in r.json["message"]
 
-    def delete_project(self, user, index, projectname, code=200):
-        r = self.testapp.delete_json("/%s/%s/%s" % (user, index,
+    def delete_project(self, projectname, code=200, indexname=None):
+        indexname = self._getindexname(indexname)
+        r = self.testapp.delete_json("/%s/%s" % (indexname,
                 projectname), {}, expect_errors=True)
         assert r.status_code == code
 
-    def upload_file_pypi(self, user, index, basename, content,
-                         name, version, code=200):
-        r = self.testapp.post("/%s/%s/" % (user, index),
+    def register_metadata(self, metadata, indexname=None, code=200):
+        indexname = self._getindexname(indexname)
+        metadata = metadata.copy()
+        metadata[":action"] = "submit"
+        r = self.testapp.post("/%s/" % indexname, metadata,
+                              expect_errors=True)
+        assert r.status_code == code
+
+    def upload_file_pypi(self, basename, content,
+                         name=None, version=None, indexname=None,
+                         register=True,
+                         code=200):
+        indexname = self._getindexname(indexname)
+        #name_version = splitbasename(basename, checkarch=False)
+        #if not name:
+        #    name = name_version[0]
+        #if not version:
+        #    version = name_version[1]
+        if register and code == 200:
+            self.register_metadata(dict(name=name, version=version))
+        r = self.testapp.post("/%s/" % indexname,
             {":action": "file_upload", "name": name, "version": version,
              "content": Upload(basename, content)}, expect_errors=True)
         assert r.status_code == code
 
 
-    def upload_doc(self, user, index, basename, content, name, version,
+    def upload_doc(self, basename, content, name, version, indexname=None,
                          code=200):
-
-        r = self.testapp.post("/%s/%s/" % (user, index),
+        indexname = self._getindexname(indexname)
+        if version:
+            self.register_metadata(dict(name=name, version=version))
+        r = self.testapp.post("/%s/" % indexname,
             {":action": "doc_upload", "name": name, "version": version,
              "content": Upload(basename, content)}, expect_errors=True)
         assert r.status_code == code
 
+    def get_simple(self, projectname, code=200):
+        r = self.testapp.get(self.api.simpleindex + projectname + "/",
+                             expect_errors=True)
+        assert r.status_code == code
+        return r
+
+
 from webtest import TestApp as TApp
 
 class MyTestApp(TApp):
@@ -328,6 +421,15 @@ class MyTestApp(TApp):
             #print ("setting auth header %r %s %s" % (auth, method, url))
         return super(MyTestApp, self)._gen_request(method, url, **kw)
 
+    def post(self, *args, **kwargs):
+        code = kwargs.pop("code", None)
+        if code is not None and code >= 300:
+            kwargs.setdefault("expect_errors", True)
+        r = super(MyTestApp, self).post(*args, **kwargs)
+        if code is not None:
+            assert r.status_code == code
+        return r
+
     def push(self, url, params=None, **kw):
         kw.setdefault("expect_errors", True)
         return self._gen_request("push", url, params=params, **kw)
@@ -347,10 +449,8 @@ class MyTestApp(TApp):
 
 
 @pytest.fixture
-def testapp(request, xom):
-    app = xom.create_app(catchall=False, immediatetasks=-1)
-    return MyTestApp(app)
-
+def testapp(request, maketestapp, xom):
+    return maketestapp(xom)
 
 ### incremental testing
 
