@@ -3,6 +3,10 @@ import py
 import os
 import pytest
 
+from devpi_server.urlutil import splitbasename
+from devpi_server.db import unzip_to_dir
+
+
 @pytest.fixture(params=[(), ("root/pypi",)])
 def bases(request):
     return request.param
@@ -11,6 +15,25 @@ def test_create_zipfile(tmpdir):
     content = create_zipfile({"one": {"nested": "1"}, "two": {}})
     tmpdir.join("hello.zip").write(content, "wb")
 
+def register_and_store(stage, basename, content="123", name=None):
+    n, version = splitbasename(basename)[:2]
+    if name is None:
+        name = n
+    stage.register_metadata(dict(name=name, version=version))
+    return stage.store_releasefile(basename, content)
+
+def test_db_is_empty(db):
+    assert db.is_empty()
+    db.user_create("user", "password", email="some@email.com")
+    assert not db.is_empty()
+    db.user_delete("user")
+    assert db.is_empty()
+    db.index_create(user="root", index="dev", bases=(), type="stage",
+                    volatile=False)
+    assert not db.is_empty()
+    db.index_delete(user="root", index="dev")
+    assert db.is_empty()
+
 class TestStage:
     @pytest.fixture
     def stage(self, request, db):
@@ -18,6 +41,7 @@ class TestStage:
                       type="stage", volatile=True)
         if "bases" in request.fixturenames:
             config["bases"] = request.getfuncargvalue("bases")
+        db.user_create("hello", "123")
         db.index_create(**config)
         return db.getstage(user=config["user"], index=config["index"])
 
@@ -80,24 +104,41 @@ class TestStage:
         assert not stage.getreleaselinks("someproject")
         assert not stage.getprojectnames()
 
-    def test_inheritance_simple(self, httpget, stage, db):
+    def test_10_metadata_name_mixup(self, stage, bases):
+        stage._register_metadata({"name": "x-encoder", "version": "1.0"})
+        key = stage.keyfs.PROJCONFIG(user=stage.user, index=stage.index,
+                                     name="x_encoder")
+        with key.locked_update() as projectconfig:
+            versionconfig = projectconfig["1.0"] = {}
+            versionconfig.update({"+files":
+                {"x_encoder-1.0.zip": "%s/x_encoder/1.0/x_encoder-1.0.zip" %
+                 stage.name}})
+        names = stage.getprojectnames_perstage()
+        assert len(names) == 2
+        assert "x-encoder" in names
+        assert "x_encoder" in names
+        # also test import/export
+        from devpi_server.importexport import Exporter
+        tw = py.io.TerminalWriter()
+        exporter = Exporter(tw, stage.xom)
+        exporter.compute_global_projectname_normalization()
+
+    def test_inheritance_simple(self, extdb, stage, db):
         stage._reconfigure(bases=("root/pypi",))
-        httpget.setextsimple("someproject",
-            "<a href='someproject-1.0.zip' /a>")
+        extdb.mock_simple("someproject", "<a href='someproject-1.0.zip' /a>")
+        assert stage.getprojectnames() == ["someproject",]
         entries = stage.getreleaselinks("someproject")
         assert len(entries) == 1
-        assert stage.getprojectnames() == ["someproject",]
         stage.register_metadata(dict(name="someproject", version="1.1"))
         assert stage.getprojectnames() == ["someproject",]
 
-    def test_inheritance_twice(self, httpget, db, stage):
+    def test_inheritance_twice(self, extdb, db, stage):
         db.index_create(user="root", index="dev2", bases=("root/pypi",))
         stage_dev2 = db.getstage("root/dev2")
         stage._reconfigure(bases=("root/dev2",))
-        httpget.setextsimple("someproject",
-            "<a href='someproject-1.0.zip' /a>")
-        stage_dev2.store_releasefile("someproject-1.1.tar.gz", "123")
-        stage.store_releasefile("someproject-1.2.tar.gz", "456")
+        extdb.mock_simple("someproject", "<a href='someproject-1.0.zip' /a>")
+        register_and_store(stage_dev2, "someproject-1.1.tar.gz")
+        register_and_store(stage_dev2, "someproject-1.2.tar.gz")
         entries = stage.getreleaselinks("someproject")
         assert len(entries) == 3
         assert entries[0].basename == "someproject-1.2.tar.gz"
@@ -105,13 +146,16 @@ class TestStage:
         assert entries[2].basename == "someproject-1.0.zip"
         assert stage.getprojectnames() == ["someproject",]
 
-    def test_inheritance_normalize_multipackage(self, httpget, db, stage):
+    def test_inheritance_normalize_multipackage(self, extdb, stage):
         stage._reconfigure(bases=("root/pypi",))
-        httpget.setextsimple("some-project", """
+        extdb.mock_simple("some-project", """
             <a href='some_project-1.0.zip' /a>
             <a href='some_project-1.0.tar.gz' /a>
         """)
-        stage.store_releasefile("some_project-1.2.tar.gz", "456")
+        register_and_store(stage, "some_project-1.2.tar.gz",
+                           name="some-project")
+        register_and_store(stage, "some_project-1.2.tar.gz",
+                           name="some-project")
         entries = stage.getreleaselinks("some-project")
         assert len(entries) == 3
         assert entries[0].basename == "some_project-1.2.tar.gz"
@@ -119,37 +163,38 @@ class TestStage:
         assert entries[2].basename == "some_project-1.0.tar.gz"
         assert stage.getprojectnames() == ["some-project",]
 
-    def test_getreleaselinks_inheritance_shadow(self, httpget, stage):
+    def test_getreleaselinks_inheritance_shadow(self, extdb, stage):
         stage._reconfigure(bases=("root/pypi",))
-        httpget.setextsimple("someproject",
+        extdb.mock_simple("someproject",
             "<a href='someproject-1.0.zip' /a>")
+        register_and_store(stage, "someproject-1.0.zip", "123")
         stage.store_releasefile("someproject-1.0.zip", "123")
         entries = stage.getreleaselinks("someproject")
         assert len(entries) == 1
         assert entries[0].relpath.endswith("someproject-1.0.zip")
 
-    def test_getreleaselinks_inheritance_shadow_egg(self, httpget, stage):
+    def test_getreleaselinks_inheritance_shadow_egg(self, extdb, stage):
         stage._reconfigure(bases=("root/pypi",))
-        httpget.setextsimple("py",
+        extdb.mock_simple("py",
         """<a href="http://bb.org/download/py.zip#egg=py-dev" />""")
-        stage.store_releasefile("py-1.0.tar.gz", "123")
+        register_and_store(stage, "py-1.0.tar.gz", "123")
         entries = stage.getreleaselinks("py")
         assert len(entries) == 2
         e0, e1 = entries
         assert e0.basename == "py.zip"
         assert e1.basename == "py-1.0.tar.gz"
 
-    def test_inheritance_error(self, httpget, stage):
+    def test_inheritance_error(self, extdb, stage):
         stage._reconfigure(bases=("root/pypi",))
-        httpget.setextsimple("someproject", status_code = -1)
+        extdb.mock_simple("someproject", status_code = -1)
         entries = stage.getreleaselinks("someproject")
         assert entries == -1
         #entries = stage.getprojectnames()
         #assert entries == -1
 
-    def test_get_projectconfig_inherited(self, httpget, stage):
+    def test_get_projectconfig_inherited(self, extdb, stage):
         stage._reconfigure(bases=("root/pypi",))
-        httpget.setextsimple("someproject",
+        extdb.mock_simple("someproject",
             "<a href='someproject-1.0.zip' /a>")
         projectconfig = stage.get_projectconfig("someproject")
         assert "someproject-1.0.zip" in projectconfig["1.0"]["+files"]
@@ -157,7 +202,7 @@ class TestStage:
     def test_store_and_get_releasefile(self, stage, bases):
         content = "123"
         content2 = "1234"
-        entry = stage.store_releasefile("some-1.0.zip", content)
+        entry = register_and_store(stage, "some-1.0.zip", content)
         entries = stage.getreleaselinks("some")
         assert len(entries) == 1
         assert entries[0].md5 == entry.md5
@@ -165,9 +210,13 @@ class TestStage:
         pconfig = stage.get_projectconfig("some")
         assert pconfig["1.0"]["+files"]["some-1.0.zip"].endswith("some-1.0.zip")
 
-    def test_project_config_shadowed(self, httpget, stage):
+    def test_store_releasefile_fails_if_not_registered(self, stage):
+        with pytest.raises(stage.MissesRegistration):
+            stage.store_releasefile("someproject-1.0.zip", "123")
+
+    def test_project_config_shadowed(self, extdb, stage):
         stage._reconfigure(bases=("root/pypi",))
-        httpget.setextsimple("someproject",
+        extdb.mock_simple("someproject",
             "<a href='someproject-1.0.zip' /a>")
         content = "123"
         entry = stage.store_releasefile("someproject-1.0.zip", content)
@@ -179,7 +228,7 @@ class TestStage:
 
     def test_store_and_delete_project(self, stage, bases):
         content = "123"
-        entry = stage.store_releasefile("some-1.0.zip", content)
+        entry = register_and_store(stage, "some-1.0.zip", content)
         pconfig = stage.get_projectconfig_perstage("some")
         assert pconfig["1.0"]
         stage.project_delete("some")
@@ -187,9 +236,8 @@ class TestStage:
         assert not pconfig
 
     def test_store_and_delete_release(self, stage, bases):
-        content = "123"
-        entry = stage.store_releasefile("some-1.0.zip", content)
-        entry = stage.store_releasefile("some-1.1.zip", content)
+        entry = register_and_store(stage, "some-1.0.zip")
+        entry = register_and_store(stage, "some-1.1.zip")
         pconfig = stage.get_projectconfig_perstage("some")
         assert pconfig["1.0"] and pconfig["1.1"]
         stage.project_version_delete("some", "1.0")
@@ -199,9 +247,8 @@ class TestStage:
         assert not stage.project_exists("some")
 
     def test_releasefile_sorting(self, stage, bases):
-        content = "123"
-        entry = stage.store_releasefile("some-1.1.zip", content)
-        entry = stage.store_releasefile("some-1.0.zip", content)
+        entry = register_and_store(stage, "some-1.1.zip")
+        entry = register_and_store(stage, "some-1.0.zip")
         entries = stage.getreleaselinks("some")
         assert len(entries) == 2
         assert entries[0].basename == "some-1.1.zip"
@@ -214,6 +261,18 @@ class TestStage:
         assert filepath.join("_static").check(dir=1)
         assert filepath.join("_templ", "x.css").check(file=1)
 
+    def test_getdoczip(self, stage, bases, tmpdir):
+        assert not stage.get_doczip("pkg1")
+        content = create_zipfile({"index.html": "<html/>",
+            "_static": {}, "_templ": {"x.css": ""}})
+        filepath = stage.store_doczip("pkg1", content)
+        doczip_content = stage.get_doczip("pkg1")
+        assert doczip_content
+        unzip_to_dir(doczip_content, tmpdir)
+        assert tmpdir.join("index.html").read() == "<html/>"
+        assert tmpdir.join("_static").check(dir=1)
+        assert tmpdir.join("_templ", "x.css").check(file=1)
+
     def test_storedoczipfile(self, stage, bases):
         content = create_zipfile({"index.html": "<html/>",
             "_static": {}, "_templ": {"x.css": ""}})
@@ -229,7 +288,7 @@ class TestStage:
         stage._reconfigure(volatile=False)
         content = "123"
         content2 = "1234"
-        entry = stage.store_releasefile("some-1.0.zip", content)
+        entry = register_and_store(stage, "some-1.0.zip", content)
         assert len(stage.getreleaselinks("some")) == 1
 
         # rewrite  fails
@@ -252,6 +311,31 @@ class TestStage:
         #stage.ixconfig["volatile"] = False
         #with pytest.raises(stage.MetadataExists):
         #    stage.register_metadata(dict(name="hello", version="1.0"))
+        #
+
+    def test_get_metadata_latest(self, stage):
+        stage.register_metadata(dict(name="hello", version="1.0"))
+        stage.register_metadata(dict(name="hello", version="1.1"))
+        stage.register_metadata(dict(name="hello", version="0.9"))
+        metadata = stage.get_metadata_latest("hello")
+        assert metadata["version"] == "1.1"
+
+    def test_releasedata_validation(self, stage):
+        with pytest.raises(ValueError):
+             stage.register_metadata( dict(name="hello_", version="1.0"))
+
+    def test_register_metadata_normalized_name_clash(self, stage):
+        stage.register_metadata(dict(name="hello-World", version="1.0"))
+        with pytest.raises(stage.RegisterNameConflict):
+            stage.register_metadata(dict(name="Hello-world", version="1.0"))
+        with pytest.raises(stage.RegisterNameConflict):
+            stage.register_metadata(dict(name="Hello_world", version="1.0"))
+
+    def test_get_existing_project(self, stage):
+        stage.register_metadata(dict(name="Hello", version="1.0"))
+        stage.register_metadata(dict(name="this", version="1.0"))
+        project = stage.get_project_info("hello")
+        assert project.name == "Hello"
 
     def test_releasedata_description(self, stage):
         source = py.std.textwrap.dedent("""\
@@ -315,6 +399,12 @@ class TestUsers:
         db.user_modify('root', email=email_address)
         assert db.user_get("root")["email"] == email_address
 
+def test_user_set_without_indexes(db):
+    db.user_create("user", "password", email="some@email.com")
+    assert db.user_exists("user")
+    db.index_create("user/hello")
+    db._user_set("user", {"password": "pass2"})
+    assert db.index_exists("user/hello")
 
 def test_setdefault_indexes(db):
     from devpi_server.main import set_default_indexes
@@ -334,7 +424,7 @@ def _writezip(zip, contentdict, prefixes=()):
         if isinstance(val, dict):
             newprefixes = prefixes + (name,)
             if not val:
-                path = os.sep.join(newprefixes) + "/"
+                path = os.sep.join(newprefixes) + os.sep
                 zipinfo = py.std.zipfile.ZipInfo(path)
                 zip.writestr(zipinfo, "")
             else:
