@@ -3,7 +3,7 @@ import py
 import pytest
 
 from devpi_server.extpypi import *
-from devpi_server.main import FatalResponse
+from devpi_server.main import FatalResponse, Fatal
 
 class TestIndexParsing:
     simplepy = DistURL("http://pypi.python.org/simple/py/")
@@ -31,6 +31,24 @@ class TestIndexParsing:
         """)
         assert len(result.releaselinks) == 3
 
+    def test_parse_index_egg_svnurl(self, monkeypatch):
+        # strange case reported by fschulze/witsch where
+        # urlparsing will yield a fragment for svn urls.
+        # it's not exactly clear how urlparse.uses_fragment
+        # sometimes contains "svn" but it's good to check
+        # that we are not sensitive to the issue.
+        import urlparse
+        monkeypatch.setattr(urlparse, "uses_fragment",
+                            urlparse.uses_fragment + ["svn"])
+        simplepy = DistURL("https://pypi.python.org/simple/zope.sqlalchemy/")
+        result = parse_index(simplepy,
+            '<a href="svn://svn.zope.org/repos/main/'
+            'zope.sqlalchemy/trunk#egg=zope.sqlalchemy-dev" />'
+        )
+        assert len(result.releaselinks) == 0
+        assert len(result.egglinks) == 0
+        #assert 0, (result.releaselinks, result.egglinks)
+
     def test_parse_index_normalized_name(self):
         simplepy = DistURL("http://pypi.python.org/simple/ndg-httpsclient/")
         result = parse_index(simplepy, """
@@ -67,6 +85,12 @@ class TestIndexParsing:
         assert result.crawllinks == \
                     set(["http://pylib.org", "http://pylib2.org"])
 
+    def test_parse_index_invalid_link(self, extdb):
+        result = parse_index(self.simplepy, '''
+                <a rel="download" href="http:/host.com/123" />
+        ''')
+        assert result.crawllinks
+
     def test_parse_index_with_egg(self):
         result = parse_index(self.simplepy,
             """<a href="http://bb.org/download/py.zip#egg=py-dev" />
@@ -87,6 +111,35 @@ class TestIndexParsing:
         link, = result.releaselinks
         assert link.basename == "py-1.0-cp27-none-linux_x86_64.whl"
 
+    @pytest.mark.parametrize("basename", [
+        "py-1.3.1.tar.gz",
+        "py-1.3.1-1.fc12.src.rpm",
+        "py-docs-1.0.zip",
+        "py-1.1.0.win-amd64.exe",
+        "py.tar.gz",
+        "py-0.8.msi",
+        "py-0.10.0.dmg",
+        "py-0.8.deb",
+        "py-12.0.0.win32-py2.7.msi",
+        "py-1.3.1-1.0rc4.tar.gz", "py-1.0.1.tar.bz2"])
+    def test_parse_index_with_valid_basenames(self, basename):
+        result = parse_index(self.simplepy, '<a href="pkg/%s" />' % basename)
+        assert len(result.releaselinks) == 1
+        link, = result.releaselinks
+        assert link.basename == basename
+
+    def test_parse_index_with_num_in_projectname(self):
+        simple = DistURL("http://pypi.python.org/simple/py-4chan/")
+        result = parse_index(simple, '<a href="pkg/py-4chan-1.0.zip"/>')
+        assert len(result.releaselinks) == 1
+        assert result.releaselinks[0].basename == "py-4chan-1.0.zip"
+
+    def test_parse_index_unparseable_url(self):
+        simple = DistURL("http://pypi.python.org/simple/x123/")
+        result = parse_index(simple, '<a href="http:" />')
+        assert len(result.releaselinks) == 0
+
+
     def test_parse_index_ftp_ignored_for_now(self):
         result = parse_index(self.simplepy,
             """<a href="http://bb.org/download/py-1.0.zip" />
@@ -115,7 +168,7 @@ class TestIndexParsing:
 
     def test_parse_index_with_matchingprojectname_no_version(self):
         result = parse_index(self.simplepy,
-            """<a href="http://bb.org/download/py.zip" />
+            """<a href="http://bb.org/download/p.zip" />
             <a href="http://bb.org/download/py-1.0.zip" />""")
         assert len(result.releaselinks) == 1
 
@@ -333,9 +386,9 @@ class TestExtPYPIDB:
         assert len(links) == 1
 
     def test_getprojectnames(self, extdb):
-        extdb.setextsimple("proj1", text='''
+        extdb.mock_simple("proj1", text='''
                            <a href="../../pkg/proj1-1.0.zip#md5=123" /> ''')
-        extdb.setextsimple("proj2", text='''
+        extdb.mock_simple("proj2", text='''
                            <a href="../../pkg/proj2-1.0.zip#md5=123" /> ''')
         extdb.url2response["https://pypi.python.org/simple/proj3/"] = dict(
             status_code=404)
@@ -345,59 +398,63 @@ class TestExtPYPIDB:
         names = extdb.getprojectnames()
         assert names == ["proj1", "proj2"]
 
+    def test_get_existing_with_302(self, extdb):
+        extdb.mock_simple("Hello_this")
+        extdb.mock_simple("hello-World")
+        extdb.mock_simple("s-p")
+        assert extdb.get_project_info("hello-this").name == "Hello_this"
+        assert extdb.get_project_info("hello_world").name == "hello-World"
+        assert extdb.get_project_info("hello-world").name == "hello-World"
+        assert extdb.get_project_info("s-p").name == "s-p"
+        assert extdb.get_project_info("s_p").name == "s-p"
 
 def raise_ValueError():
     raise ValueError(42)
 
 class TestRefreshManager:
 
-    def test_pypichanges_nochanges(self, extdb, keyfs):
+    def test_init_pypi_mirror(self, extdb, keyfs):
         proxy = mock.create_autospec(XMLProxy)
-        proxy.list_packages_with_serial.return_value = {"hello": 10,
-                                                        "abc": 42}
-        proxy.changelog_since_serial.return_value = []
-        with pytest.raises(ValueError):
-            extdb.spawned_pypichanges(proxy, proxysleep=raise_ValueError)
-        proxy.list_packages_with_serial.assert_called_once_with()
-        assert keyfs.PYPISERIALS.get()["hello"] == 10
-        assert keyfs.PYPISERIALS.get()["abc"] == 42
+        d = {"hello": 10, "abc": 42}
+        proxy.list_packages_with_serial.return_value = d
+        extdb.init_pypi_mirror(proxy)
+        assert extdb.name2serials == d
+        assert keyfs.PYPISERIALS.get() == d
         assert extdb.getprojectnames() == ["abc", "hello"]
 
-    def test_pypichanges_mirrorversionchange(self, extdb, keyfs, monkeypatch):
+    def test_pypichanges_loop(self, extdb, monkeypatch):
+        extdb.process_changelog = mock.Mock()
+        extdb.process_refreshes = mock.Mock()
         proxy = mock.create_autospec(XMLProxy)
-        proxy.list_packages_with_serial.return_value = {"hello": 10}
-        proxy.changelog_since_serial.return_value = []
-        with pytest.raises(ValueError):
-            extdb.spawned_pypichanges(proxy, proxysleep=raise_ValueError)
-        proxy.list_packages_with_serial.assert_called_once_with()
-
-        # trying again should not trigger restart
-        proxy.list_packages_with_serial.called = False
-        with pytest.raises(ValueError):
-            extdb.spawned_pypichanges(proxy, proxysleep=raise_ValueError)
-        assert not proxy.list_packages_with_serial.called
-        monkeypatch.setattr(extdb, "MIRRORVERSION", extdb.MIRRORVERSION + 1)
-        # trying again should trigger restart
-        with pytest.raises(ValueError):
-            extdb.spawned_pypichanges(proxy, proxysleep=raise_ValueError)
-        assert proxy.list_packages_with_serial.called
-
-
-    def test_pypichanges_changes(self, extdb, httpget, keyfs, monkeypatch):
-        keyfs.PYPISERIALS.set({"pytest": 20, ".ver": extdb.MIRRORVERSION})
-        httpget.setextsimple("pytest", '<a href="pytest-2.3.tgz"/a>',
-                             pypiserial=20)
-        assert len(extdb.getreleaselinks("pytest")) == 1
-        proxy = mock.create_autospec(XMLProxy)
-        proxy.changelog_since_serial.return_value = [
+        changelog = [
             ["pylib", "1.4", 12123, 'new release', 11],
-            ["pytest", "2.4", 121231, 'new release', 27]]
-        httpget.setextsimple("pytest", '<a href="pytest-2.4.tgz"/a>',
-                             pypiserial=27)
+            ["pytest", "2.4", 121231, 'new release', 27]
+        ]
+        proxy.changelog_since_serial.return_value = changelog
+
+        # we need to have one entry in serials
+        extdb.mock_simple("pytest", pypiserial=27)
         with pytest.raises(ValueError):
             extdb.spawned_pypichanges(proxy, proxysleep=raise_ValueError)
-        assert not proxy.list_packages_with_serial.called
+        extdb.process_changelog.assert_called_once_with(changelog)
+        extdb.process_refreshes.assert_called_once()
+
+    def test_pypichanges_changes(self, extdb, keyfs, monkeypatch):
+        assert not extdb.name2serials
+        extdb.mock_simple("pytest", '<a href="pytest-2.3.tgz"/a>',
+                          pypiserial=20)
+        assert len(extdb.name2serials) == 1
+        assert len(extdb.getreleaselinks("pytest")) == 1
+        extdb.process_changelog([
+            ["Django", "1.4", 12123, 'new release', 11],
+            ["pytest", "2.4", 121231, 'new release', 27]
+        ])
+        assert len(extdb.name2serials) == 2
         assert keyfs.PYPISERIALS.get()["pytest"] == 27
+        assert keyfs.PYPISERIALS.get()["Django"] == 11
+        extdb.mock_simple("pytest", '<a href="pytest-2.4.tgz"/a>',
+                          pypiserial=27)
+        extdb.process_refreshes()
         assert extdb.getreleaselinks("pytest")[0].basename == "pytest-2.4.tgz"
 
     @pytest.fixture(params=["protocol", "socket"])
@@ -413,10 +470,10 @@ class TestRefreshManager:
         return raise_error
 
     def test_changelog_since_serial_nonetwork(self, extdb,
-                    keyfs, raise_error, monkeypatch, caplog):
+                    raise_error, monkeypatch, caplog):
+        extdb.mock_simple("pytest", pypiserial=10)
         from xmlrpclib import ServerProxy
         got = []
-        keyfs.PYPISERIALS.set({"pytest": 10, ".ver": extdb.MIRRORVERSION})
         def raise_xmlrpcish(since_int):
             got.append(since_int)
             raise_error()
@@ -430,24 +487,12 @@ class TestRefreshManager:
         assert got == [10,10]
         assert caplog.getrecords(".*since_serial.*error.*")
 
-    def test_changelog_list_packages_no_network(self, extdb,
-            keyfs, raise_error, monkeypatch, caplog):
-        from xmlrpclib import ProtocolError, ServerProxy
-
-        proxyanswers = [None]
-        loops = []
-        def proxysleep():
-            loops.append(1)
-            if not proxyanswers.pop():
-                raise ValueError
-
-        serverproxy = mock.Mock()
-        serverproxy.list_packages_with_serial.side_effect = raise_ValueError
-        xmlproxy = XMLProxy(serverproxy)
-        with pytest.raises(ValueError):
-            extdb.spawned_pypichanges(xmlproxy, proxysleep=raise_ValueError)
-        assert not keyfs.PYPISERIALS.exists()
-        assert caplog.getrecords(".*error.*")
+    def test_changelog_list_packages_no_network(self, makexom):
+        xmlproxy = mock.create_autospec(XMLProxy)
+        xmlproxy.list_packages_with_serial.return_value = None
+        with pytest.raises(Fatal):
+            xom = makexom(proxy=xmlproxy)
+        #assert not xom.keyfs.PYPISERIALS.exists()
 
 
 def test_requests_httpget_negative_status_code(xom_notmocked, monkeypatch):
@@ -469,3 +514,12 @@ def test_requests_httpget_timeout(xom_notmocked, monkeypatch):
     r = xom_notmocked.httpget("http://notexists.qwe", allow_redirects=False,
                               timeout=1.2)
     assert r.status_code == -1
+
+def test_invalidate_on_version_change(tmpdir, caplog):
+    from devpi_server.extpypi import invalidate_on_version_change, ExtDB
+    p = tmpdir.ensure("something")
+    invalidate_on_version_change(tmpdir)
+    assert not p.check()
+    assert tmpdir.join(".mirrorversion").read() == ExtDB.VERSION
+    rec, = caplog.getrecords()
+    assert "format change" in rec.msg
