@@ -1,10 +1,12 @@
 
 import pytest
+import re
 import py
 import webtest
 import requests, json
 from bs4 import BeautifulSoup
 from devpi_server.views import *
+from devpi_server.urlutil import splitbasename
 import devpi_server.views
 from test_db import create_zipfile
 from mock import Mock
@@ -15,24 +17,30 @@ from .functional import TestUserThings, TestIndexThings
 def getfirstlink(text):
     return BeautifulSoup(text).findAll("a")[0]
 
-def test_simple_project(pypiurls, httpget, testapp):
+def test_simple_project(pypiurls, extdb, testapp):
     name = "qpwoei"
     r = testapp.get("/root/pypi/+simple/" + name)
-    assert r.status_code == 404
+    assert r.status_code == 200
+    assert not BeautifulSoup(r.text).findAll("a")
     path = "/%s-1.0.zip" % name
-    httpget.setextsimple(name, text='<a href="%s"/>' % path)
+    extdb.mock_simple(name, text='<a href="%s"/>' % path)
     r = testapp.get("/root/pypi/+simple/%s" % name)
     assert r.status_code == 200
     links = BeautifulSoup(r.text).findAll("a")
     assert len(links) == 1
     assert links[0].get("href").endswith(path)
 
+def test_simple_project_unicode_rejected(pypiurls, extdb, testapp):
+    name = "qpw\xc3"
+    r = testapp.get("/root/pypi/+simple/" + name)
+    assert r.status_code == 400
+
 def test_simple_url_longer_triggers_404(testapp):
     assert testapp.get("/root/pypi/+simple/pytest/1.0/").status_code == 404
     assert testapp.get("/root/pypi/+simple/pytest/1.0").status_code == 404
 
-def test_simple_project_pypi_egg(pypiurls, httpget, testapp):
-    httpget.setextsimple("py",
+def test_simple_project_pypi_egg(pypiurls, extdb, testapp):
+    extdb.mock_simple("py",
         """<a href="http://bb.org/download/py.zip#egg=py-dev" />""")
     r = testapp.get("/root/pypi/+simple/py/")
     assert r.status_code == 200
@@ -41,12 +49,14 @@ def test_simple_project_pypi_egg(pypiurls, httpget, testapp):
     r = testapp.get("/root/pypi/")
     assert r.status_code == 200
 
-def test_simple_list(pypiurls, httpget, testapp):
-    httpget.setextsimple("hello1", text="<html/>")
-    httpget.setextsimple("hello2", text="<html/>")
+def test_simple_list(pypiurls, extdb, testapp):
+    extdb.mock_simple("hello1", "<html/>")
+    extdb.mock_simple("hello2", "<html/>")
     assert testapp.get("/root/pypi/+simple/hello1").status_code == 200
     assert testapp.get("/root/pypi/+simple/hello2").status_code == 200
-    assert testapp.get("/root/pypi/+simple/hello3").status_code == 404
+    r = testapp.get("/root/pypi/+simple/hello3")
+    assert r.status_code == 200
+    assert "no such project" in r.text
     r = testapp.get("/root/pypi/+simple/")
     assert r.status_code == 200
     links = BeautifulSoup(r.text).findAll("a")
@@ -54,20 +64,26 @@ def test_simple_list(pypiurls, httpget, testapp):
     hrefs = [a.get("href") for a in links]
     assert hrefs == ["hello1/", "hello2/"]
 
-def test_indexroot(pypiurls, httpget, testapp, xom):
+def test_indexroot(pypiurls, testapp, xom):
     xom.db.create_stage("user/index", bases=("root/pypi",))
     r = testapp.get("/user/index/")
     assert r.status_code == 200
 
+def test_indexroot_root_pypi(testapp, xom):
+    r = testapp.get("/root/pypi/")
+    assert r.status_code == 200
+    assert "in-stage" not in r.body
+
+
 @pytest.mark.parametrize("code", [-1, 500, 501, 502, 503])
-def test_upstream_not_reachable(pypiurls, httpget, testapp, xom, code):
+def test_upstream_not_reachable(pypiurls, extdb, testapp, xom, code):
     name = "whatever%d" % (code + 1)
-    httpget.setextsimple(name, status_code = code)
+    extdb.mock_simple(name, status_code = code)
     r = testapp.get("/root/pypi/+simple/%s" % name)
     assert r.status_code == 502
 
-def test_pkgserv(pypiurls, httpget, testapp):
-    httpget.setextsimple("package", '<a href="/package-1.0.zip" />')
+def test_pkgserv(pypiurls, httpget, extdb, testapp):
+    extdb.mock_simple("package", '<a href="/package-1.0.zip" />')
     httpget.setextfile("/package-1.0.zip", "123")
     r = testapp.get("/root/pypi/+simple/package/")
     assert r.status_code == 200
@@ -75,7 +91,7 @@ def test_pkgserv(pypiurls, httpget, testapp):
     r = testapp.get(href)
     assert r.body == "123"
 
-def test_apiconfig(httpget, testapp):
+def test_apiconfig(testapp):
     r = testapp.get("/user/name/+api")
     assert r.status_code == 404
     r = testapp.get("/root/pypi/+api")
@@ -86,14 +102,12 @@ def test_apiconfig(httpget, testapp):
     #    assert name in r.json
     #
     #
-def test_root_pypi(httpget, testapp):
+def test_root_pypi(testapp):
     r = testapp.get("/root/pypi/")
     assert r.status_code == 200
 
-def test_register_metadata_and_get_description(httpget, db, mapp, testapp):
-    mapp.create_and_login_user("user")
-    mapp.create_index("name")
-    api = mapp.getapi("user/name")
+def test_register_metadata_and_get_description(mapp, testapp):
+    api = mapp.create_and_use("user/name")
     metadata = {"name": "pkg1", "version": "1.0", ":action": "submit",
                 "description": "hello world"}
     r = testapp.post(api.pypisubmit, metadata)
@@ -105,7 +119,68 @@ def test_register_metadata_and_get_description(httpget, db, mapp, testapp):
     assert r.status_code == 200
     assert "1.0" in r.json["result"]
 
-def test_push_non_existent(httpget, db, mapp, testapp, monkeypatch):
+class TestSubmitValidation:
+    @pytest.fixture
+    def submit(self, mapp, testapp):
+        class Submit:
+            def __init__(self, stagename="user/dev"):
+                self.stagename = stagename
+                self.api = mapp.create_and_use(stagename)
+
+            def metadata(self, metadata, code):
+                return testapp.post(self.api.pypisubmit, metadata, code=code)
+
+            def file(self, filename, content, metadata, code=200):
+                if "version" not in metadata:
+                    metadata["version"] = splitbasename(filename,
+                                                        checkarch=False)[1]
+                return mapp.upload_file_pypi(
+                        filename, content,
+                        metadata.get("name"), metadata.get("version"),
+                        indexname=self.stagename,
+                        code=code)
+        return Submit()
+
+    def test_metadata_normalize_conflict(self, submit, testapp):
+        metadata = {"name": "pKg1", "version": "1.0", ":action": "submit",
+                    "description": "hello world"}
+        r = submit.metadata(metadata, code=200)
+        metadata = {"name": "Pkg1", "version": "1.0", ":action": "submit",
+                    "description": "hello world"}
+        r = submit.metadata(metadata, code=403)
+        assert re.search("pKg1.*already.*registered", r.body)
+
+    def test_metadata_normalize_inheritance(self, extdb, submit):
+        extdb.mock_simple("pKg1")
+        metadata = {"name": "Pkg1", "version": "1.0", ":action": "submit",
+                    "description": "hello world"}
+        r = submit.metadata(metadata, code=403)
+        assert re.search("Pkg1.*because.*pKg1.*already.*registered.*pypi",
+                         r.body)
+
+    def test_upload_file(self, submit):
+        metadata = {"name": "Pkg5", "version": "1.0", ":action": "submit"}
+        submit.metadata(metadata, code=200)
+        submit.file("pkg5-2.6.tgz", "123", {"name": "pkg5some"}, code=400)
+        submit.file("pkg5-2.6.tgz", "123", {"name": "Pkg5"}, code=200)
+        submit.file("pkg5-2.6.qwe", "123", {"name": "Pkg5"}, code=400)
+        submit.file("pkg5-2.7.tgz", "123", {"name": "pkg5"}, code=403)
+
+    def test_upload_and_simple_index(self, submit, testapp):
+        metadata = {"name": "Pkg5", "version": "2.6", ":action": "submit"}
+        submit.metadata(metadata, code=200)
+        submit.file("pkg5-2.6.tgz", "123", {"name": "Pkg5"}, code=200)
+        r = testapp.get("/%s/+simple/pkg5" % submit.stagename)
+        assert r.status_code == 302
+
+    def test_get_project_redirected(self, submit, mapp):
+        metadata = {"name": "Pkg1", "version": "1.0", ":action": "submit",
+                    "description": "hello world"}
+        submit.metadata(metadata, code=200)
+        location = mapp.getjson("/%s/pkg1" % submit.stagename, code=302)
+        assert location.endswith("/Pkg1/")
+
+def test_push_non_existent(mapp, testapp, monkeypatch):
     # check that push from non-existent index results in 404
     req = dict(name="pkg5", version="2.6", targetindex="user2/dev")
     r = testapp.push("/user2/dev/", json.dumps(req), expect_errors=True)
@@ -124,23 +199,24 @@ def test_push_non_existent(httpget, db, mapp, testapp, monkeypatch):
     r = testapp.push("/user1/dev/", json.dumps(req), expect_errors=True)
     assert r.status_code == 404
     #
-    mapp.upload_file_pypi(
-            "user1", "dev", "pkg5-2.6.tgz", "123", "pkg5", "2.6")
+    mapp.use("user1/dev")
+    mapp.upload_file_pypi("pkg5-2.6.tgz", "123", "pkg5", "2.6")
     # check that push to non-authoried existent target index results in 401
     r = testapp.push("/user1/dev/", json.dumps(req), expect_errors=True)
     assert r.status_code == 401
 
-def test_upload_and_push_internal(httpget, db, mapp, testapp, monkeypatch):
+def test_upload_and_push_internal(mapp, testapp, monkeypatch):
     mapp.create_user("user1", "1")
     mapp.create_and_login_user("user2")
     mapp.create_index("prod", indexconfig=dict(acl_upload=["user1", "user2"]))
     mapp.create_index("dev", indexconfig=dict(acl_upload=["user2"]))
-    print mapp.getjson("/user2/prod")
 
     mapp.login("user1", "1")
     mapp.create_index("dev")
-    mapp.upload_file_pypi(
-            "user1", "dev", "pkg1-2.6.tgz", "123", "pkg1", "2.6")
+    mapp.use("user1/dev")
+    mapp.upload_file_pypi("pkg1-2.6.tgz", "123", "pkg1", "2.6")
+    content = create_zipfile({"index.html": "<html/>"})
+    mapp.upload_doc("pkg1.zip", content, "pkg1", "")
 
     # check that push is authorized and executed towards user2/prod index
     req = dict(name="pkg1", version="2.6", targetindex="user2/prod")
@@ -150,19 +226,23 @@ def test_upload_and_push_internal(httpget, db, mapp, testapp, monkeypatch):
     assert r.status_code == 200
     relpath = r.json["result"]["+files"]["pkg1-2.6.tgz"]
     assert relpath == "user2/prod/pkg1/2.6/pkg1-2.6.tgz"
+    r = testapp.get("/user2/prod/pkg1/+doc/index.html")
+    assert r.status_code == 200
 
-def test_upload_and_push_remote_ok(httpget, db, mapp, testapp, monkeypatch):
-    mapp.create_and_login_user("user")
-    mapp.create_index("name")
-    mapp.upload_file_pypi(
-            "user", "name", "pkg1-2.6.tgz", "123", "pkg1", "2.6")
-    r = testapp.get("/user/name/+simple/pkg1/")
+
+def test_upload_and_push_external(mapp, testapp, monkeypatch):
+    api = mapp.create_and_use()
+    mapp.upload_file_pypi("pkg1-2.6.tgz", "123", "pkg1", "2.6")
+    zipcontent = create_zipfile({"index.html": "<html/>"})
+    mapp.upload_doc("pkg1.zip", zipcontent, "pkg1", "")
+
+    r = testapp.get(api.simpleindex + "pkg1")
     assert r.status_code == 200
     a = getfirstlink(r.text)
     assert "pkg1-2.6.tgz" in a.get("href")
 
     # get root index page
-    r = testapp.get("/user/name/")
+    r = testapp.get(api.index)
     assert r.status_code == 200
 
     # push
@@ -176,12 +256,16 @@ def test_upload_and_push_remote_ok(httpget, db, mapp, testapp, monkeypatch):
         return r
     monkeypatch.setattr(requests, "post", recpost)
     body = json.dumps(req)
-    r = testapp.request("/user/name/", method="push", body=body,
+    r = testapp.request(api.index, method="push", body=body,
                         expect_errors=True)
     assert r.status_code == 200
-    assert len(rec) == 2
+    assert len(rec) == 3
     assert rec[0][0] == "whatever"
     assert rec[1][0] == "whatever"
+    assert rec[2][0] == "whatever"
+    upload_dict = rec[2][-1]
+    assert upload_dict["content"][0] == "pkg1.zip"
+    assert upload_dict["content"][1] == zipcontent
 
     # push with error
     def posterror(url, data, auth, files=None):
@@ -189,19 +273,17 @@ def test_upload_and_push_remote_ok(httpget, db, mapp, testapp, monkeypatch):
             status_code = 500
         return r
     monkeypatch.setattr(requests, "post", posterror)
-    r = testapp.request("/user/name/", method="push", body=body,
+    r = testapp.request(api.index, method="push", body=body,
                         expect_errors=True)
     assert r.status_code == 502
     result = r.json["result"]
     assert len(result) == 1
     assert result[0][0] == 500
 
-def test_upload_and_push_egg(httpget, db, mapp, testapp, monkeypatch):
-    mapp.create_and_login_user("user")
-    mapp.create_index("name")
-    mapp.upload_file_pypi(
-            "user", "name", "pkg2-1.0-py27.egg", "123", "pkg2", "1.0")
-    r = testapp.get("/user/name/+simple/pkg2/")
+def test_upload_and_push_egg(mapp, testapp, monkeypatch):
+    api = mapp.create_and_use()
+    mapp.upload_file_pypi("pkg2-1.0-py27.egg", "123", "pkg2", "1.0")
+    r = testapp.get(api.simpleindex + "pkg2/")
     assert r.status_code == 200
     a = getfirstlink(r.text)
     assert "pkg2-1.0-py27.egg" in a.get("href")
@@ -216,145 +298,130 @@ def test_upload_and_push_egg(httpget, db, mapp, testapp, monkeypatch):
             status_code = 200
         return r
     monkeypatch.setattr(requests, "post", recpost)
-    r = testapp.push("/user/name/", json.dumps(req))
+    r = testapp.push(api.index, json.dumps(req))
     assert r.status_code == 200
     assert len(rec) == 2
     assert rec[0][0] == "whatever"
     assert rec[1][0] == "whatever"
 
-def test_upload_and_delete_project(httpget, db, mapp, testapp, monkeypatch):
-    mapp.create_and_login_user("user")
-    mapp.create_index("name", indexconfig=dict(volatile=True))
-    mapp.delete_project("user", "name", "pkg1", code=404)
-    mapp.upload_file_pypi(
-            "user", "name", "pkg1-2.6.tgz", "123", "pkg1", "2.6")
-    r = testapp.get("/user/name/+simple/pkg1/")
+def test_upload_and_delete_project(mapp, testapp):
+    api = mapp.create_and_use()
+    mapp.delete_project("pkg1", code=404)
+    mapp.upload_file_pypi("pkg1-2.6.tgz", "123", "pkg1", "2.6")
+    r = testapp.get(api.simpleindex + "pkg1/")
     assert r.status_code == 200
-    r = testapp.delete("/user/name/pkg1/")
+    r = testapp.delete(api.index + "pkg1/")
     assert r.status_code == 200
-    data = mapp.getjson("/user/name/pkg1/")
+    data = mapp.getjson(api.index + "pkg1/")
     #assert data["status"] == 404
     assert not data["result"]
 
-def test_upload_with_acl(httpget, db, mapp, testapp, monkeypatch):
+def test_upload_with_acl(mapp):
     mapp.login("root")
     mapp.change_password("root", "123")
     mapp.create_user("user", "123")
-
-    mapp.create_and_login_user("i1", "456")
-    mapp.create_index("dev")
-
+    api = mapp.create_and_use()  # new context and login
     mapp.login("user", "123")
     # user cannot write to index now
-    mapp.upload_file_pypi(
-            "i1", "dev", "pkg1-2.6.tgz", "123", "pkg1", "2.6", code=403)
-    mapp.login("i1", "456")
-    mapp.set_acl("i1/dev", ["user"])
+    mapp.upload_file_pypi("pkg1-2.6.tgz", "123", "pkg1", "2.6", code=403)
+    mapp.login(api.user, api.password)
+    mapp.set_acl(["user"])
     mapp.login("user", "123")
-    mapp.upload_file_pypi(
-            "i1", "dev", "pkg1-2.6.tgz", "123", "pkg1", "2.6")
+    mapp.upload_file_pypi("pkg1-2.6.tgz", "123", "pkg1", "2.6")
 
-def test_upload_with_jenkins(db, mapp, testapp, monkeypatch):
-    mapp.create_and_login_user("i1", "456")
-    mapp.create_index("dev")
-    mapp.set_uploadtrigger_jenkins("i1/dev", "http://x.com/{pkgname}")
+def test_upload_with_jenkins(mapp, monkeypatch):
+    mapp.create_and_use()
+    mapp.set_uploadtrigger_jenkins("http://x.com/{pkgname}")
     from devpi_server import views
     post_mock = Mock(autospec=requests.post)
     monkeypatch.setattr(views.requests, "post", post_mock)
-    mapp.upload_file_pypi(
-            "i1", "dev", "pkg1-2.6.tgz", "123", "pkg1", "2.6", code=200)
+    mapp.upload_file_pypi("pkg1-2.6.tgz", "123", "pkg1", "2.6", code=200)
     assert post_mock.call_count == 1
     args = post_mock.call_args
     assert args[0][0] == "http://x.com/pkg1"
     assert args[1]["data"]["Submit"] == "Build"
 
-def test_upload_and_testdata(db, mapp, testapp, monkeypatch):
-    api = mapp.getapi()
+def test_upload_and_testdata(mapp, testapp):
     from test_devpi_server.example import tox_result_data
-    mapp.create_and_login_user("i1", "456")
-    mapp.create_index("dev")
-    mapp.upload_file_pypi(
-            "i1", "dev", "pkg1-2.6.tgz", "123", "pkg1", "2.6", code=200)
+    api = mapp.create_and_use()
+    mapp.upload_file_pypi("pkg1-2.6.tgz", "123", "pkg1", "2.6", code=200)
     r = testapp.post_json(api.resultlog, tox_result_data)
     path = r.json["result"]
     assert r.status_code == 200
     r = testapp.get(path)
     assert r.status_code == 200
 
-def test_upload_and_delete_project_version(httpget, db,
-                                           mapp, testapp, monkeypatch):
-    mapp.create_and_login_user("user")
-    mapp.create_index("name")
-    r = testapp.delete("/user/name/pkg1/", expect_errors=True)
-    assert r.status_code == 404
-    mapp.upload_file_pypi(
-            "user", "name", "pkg1-2.6.tgz", "123", "pkg1", "2.6")
-    mapp.upload_file_pypi(
-            "user", "name", "pkg1-2.7.tgz", "123", "pkg1", "2.7")
-    r = testapp.get("/user/name/+simple/pkg1/")
-    assert r.status_code == 200
-    r = testapp.delete("/user/name/pkg1/1.0", expect_errors=True)
-    assert r.status_code == 404
-    r = testapp.delete("/user/name/pkg1/2.6/")
-    assert r.status_code == 200
-    assert mapp.getjson("/user/name/pkg1/")["result"]
-    assert testapp.delete("/user/name/pkg1/2.7").status_code == 200
-    #assert mapp.getjson("/user/name/pkg1/")["status"] == 404
-    assert not mapp.getjson("/user/name/pkg1/")["result"]
-
-def test_delete_version_fails_on_non_volatile(httpget, db,
-                                              mapp, testapp, monkeypatch):
-    mapp.create_and_login_user("user")
-    mapp.create_index("name", indexconfig=dict(volatile=False))
-    mapp.upload_file_pypi(
-            "user", "name", "pkg1-2.6.tgz", "123", "pkg1", "2.6")
-    r = testapp.delete("/user/name/pkg1/2.6", expect_errors=True)
-    assert r.status_code == 403
-
-def test_upload_and_access_releasefile_meta(httpget, db,
-                                           mapp, testapp, monkeypatch):
-    mapp.create_and_login_user("user")
-    mapp.create_index("name")
-    mapp.upload_file_pypi(
-            "user", "name", "pkg5-2.6.tgz", "123", "pkg1", "2.6")
-    json = mapp.getjson("/user/name/pkg5/")
+def test_upload_and_access_releasefile_meta(mapp):
+    api = mapp.create_and_use()
+    mapp.upload_file_pypi("pkg5-2.6.tgz", "123", "pkg5", "2.6")
+    json = mapp.getjson(api.index + "pkg5/")
     href = json["result"]["2.6"]["+files"].values()[0]
     pkgmeta = mapp.getjson("/" + href)
     assert pkgmeta["type"] == "releasefilemeta"
     assert pkgmeta["result"]["size"] == "3"
 
-def test_upload_pypi_fails(httpget, db, mapp, testapp):
+def test_upload_and_delete_project_version(mapp):
+    api = mapp.create_and_use()
+    mapp.delete_project("pkg1", code=404)
+    mapp.upload_file_pypi("pkg1-2.6.tgz", "123", "pkg1", "2.6")
+    mapp.upload_file_pypi("pkg1-2.7.tgz", "123", "pkg1", "2.7")
+    mapp.get_simple("pkg1", code=200)
+    mapp.delete_project("pkg1/1.0", code=404)
+    mapp.delete_project("pkg1/2.6", code=200)
+    assert mapp.getjson(api.index + "pkg1/")["result"]
+    mapp.delete_project("pkg1/2.7", code=200)
+    #assert mapp.getjson("/user/name/pkg1/")["status"] == 404
+    assert not mapp.getjson(api.index + "pkg1/")["result"]
+
+def test_delete_version_fails_on_non_volatile(mapp):
+    api = mapp.create_and_use(indexconfig=dict(volatile=False))
+    mapp.upload_file_pypi("pkg1-2.6.tgz", "123", "pkg1", "2.6")
+    mapp.delete_project("pkg1/2.6", code=403)
+
+
+def test_upload_pypi_fails(mapp):
     mapp.upload_file_pypi(
-            "root", "pypi", "pkg1-2.6.tgz", "123", "pkg1", "2.6", code=404)
+            "pkg1-2.6.tgz", "123", "pkg1", "2.6", code=404,
+            indexname="root/pypi")
 
-def test_delete_pypi_fails(httpget, db, mapp, testapp):
+def test_delete_pypi_fails(mapp):
     mapp.login_root()
-    r = testapp.delete("/root/pypi/pytest/2.3.5", expect_errors=True)
-    assert r.status_code == 405
-    r = testapp.delete("/root/pypi/pytest", expect_errors=True)
-    assert r.status_code == 405
+    mapp.use("root/pypi")
+    mapp.delete_project("pytest/2.3.5", code=405)
+    mapp.delete_project("pytest", code=405)
 
-def test_delete_volatile_fails(httpget, db, mapp, testapp):
+def test_delete_volatile_fails(mapp):
     mapp.login_root()
     mapp.create_index("test", indexconfig=dict(volatile=False))
-    mapp.upload_file_pypi(
-            "root", "test", "pkg5-2.6.tgz", "123", "pkg1", "2.6")
-    mapp.delete_project("root", "test", "pkg5", code=403)
+    mapp.use("root/test")
+    mapp.upload_file_pypi("pkg5-2.6.tgz", "123", "pkg5", "2.6")
+    mapp.delete_project("pkg5", code=403)
 
-def test_upload_docs_too_large(httpget, db, mapp, testapp):
-    from devpi_server.views import MAXDOCZIPSIZE
-    mapp.create_and_login_user("user")
-    mapp.create_index("name")
-    content = "*" * (MAXDOCZIPSIZE + 1)
-    mapp.upload_doc("user", "name", "pkg1.zip", content, "pkg1", "2.6",
-                    code=413)
-
-def test_upload_docs(httpget, db, mapp, testapp):
-    mapp.create_and_login_user("user")
-    mapp.create_index("name")
+def test_upload_docs_no_version(mapp, testapp):
+    api = mapp.create_and_use()
     content = create_zipfile({"index.html": "<html/>"})
-    mapp.upload_doc("user", "name", "pkg1.zip", content, "pkg1", "2.6")
-    r = testapp.get("/user/name/pkg1/+doc/index.html")
+    mapp.register_metadata(dict(name="Pkg1", version="1.0"))
+    mapp.upload_doc("pkg1.zip", content, "Pkg1", "")
+    r = testapp.get(api.index + "Pkg1/+doc/index.html")
+    assert r.status_code == 200
+
+def test_upload_docs_no_project_ever_registered(mapp, testapp):
+    api = mapp.create_and_use()
+    content = create_zipfile({"index.html": "<html/>"})
+    mapp.upload_doc("pkg1.zip", content, "pkg1", "", code=400)
+
+def test_upload_docs_too_large(mapp):
+    from devpi_server.views import MAXDOCZIPSIZE
+    mapp.create_and_use()
+    content = "*" * (MAXDOCZIPSIZE + 1)
+    mapp.register_metadata(dict(name="pkg1", version="0.0"))
+    mapp.upload_doc("pkg1.zip", content, "pkg1", "2.6", code=413)
+
+def test_upload_docs(mapp, testapp):
+    api = mapp.create_and_use()
+    content = create_zipfile({"index.html": "<html/>"})
+    mapp.upload_doc("pkg1.zip", content, "pkg1", "2.6")
+    r = testapp.get(api.index + "pkg1/+doc/index.html")
     assert r.status_code == 200
     #a = getfirstlink(r.text)
     #assert "pkg1-2.6.tgz" in a.get("href")
@@ -427,39 +494,3 @@ class Test_getjson:
         assert abort_calls[0][0][0] == 400
 
 
-class TestAuth:
-    @pytest.fixture
-    def auth(self, db):
-        from devpi_server.views import Auth
-        return Auth(db, "qweqwe")
-
-    def test_no_auth(self, auth):
-        assert auth.get_auth_user(None) is None
-
-    def test_auth_direct(self, db, auth):
-        db.user_create("user", password="world")
-        assert auth.get_auth_user(("user", "world")) == "user"
-
-    def test_proxy_auth(self, db, auth):
-        db.user_create("user", password="world")
-        assert auth.new_proxy_auth("user", "wrongpass") is None
-        assert auth.new_proxy_auth("uer", "wrongpass") is None
-        res = auth.new_proxy_auth("user", "world")
-        assert "password" in res and "expiration" in res
-        assert auth.get_auth_user(("user", res["password"]))
-
-    def test_proxy_auth_expired(self, db, auth, monkeypatch):
-        user, password = "user", "world"
-
-        db.user_create(user, password)
-        proxy = auth.new_proxy_auth(user, password)
-
-        def r(*args): raise py.std.itsdangerous.SignatureExpired("123")
-        monkeypatch.setattr(auth.signer, "unsign", r)
-
-        newauth = (user, proxy["password"])
-        res = auth.get_auth_user(newauth, raising=False)
-        assert res is None
-        with pytest.raises(auth.Expired):
-            auth.get_auth_user(newauth)
-
